#==========================================================
# Aggregates --
#
#   handling of PostgreSQL Aggregates
#==========================================================
#
namespace eval Aggregates {}


#----------------------------------------------------------
# ::Aggregates::new --
#
#   sets up to create a new Aggregate
#
# Arguments:
#   none
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Aggregates::new {} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Aggregates::new


#----------------------------------------------------------
# ::Aggregates::open --
#
#   passes work to design proc for opening a Aggregate
#
# Arguments:
#   aggregate_    name of Aggregate to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Aggregates::open {aggregate_} {

    ::Aggregates::design $aggregate_

}; # end proc ::Aggregates::open


#----------------------------------------------------------
# ::Aggregates::design --
#
#   opens window in design mode for selected Aggregate
#
# Arguments:
#   aggregate_    name of Aggregate to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Aggregates::design {aggregate_} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Aggregates::new


#----------------------------------------------------------
# ::Aggregates::introspect --
#
#   Given a aggregatename, returns the SQL needed to recreate it
#
# Arguments:
#   aggname_    name of a aggregate to introspect
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to make this aggregate
#----------------------------------------------------------
#
proc ::Aggregates::introspect {aggname_ {dbh_ ""}} {

    set insql [::Aggregates::clone $aggname_ $aggname_ $dbh_]

    return $insql

}; # end proc ::Aggregates::introspect


#----------------------------------------------------------
# ::Aggregates::clone --
#
#   Like introspect, only changes the aggregatename.
#   Borrows heavily from pg_dump.c internals.
#
# Arguments:
#   srcagg_     the original aggregate, which we expect in
#               the format:
#                   namespace.aggname.xfertype.xferfunction
#   dstagg_     the clone aggregate, just aggname please
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to clone this aggregate
#----------------------------------------------------------
#
proc ::Aggregates::clone {srcagg_ dstagg_ {dbh_ ""}} {

    global CurrentDB

    if {[string match "" $dbh_]} {
        set dbh_ $CurrentDB
    }

    set V [::Database::getPgVersion $dbh_]

    set insql ""

    set srcagg_ [string map {\" {}} $srcagg_]
    set dstagg_ [string map {\" {}} $dstagg_]

    # we have to butcher this because aggregate names are
    # NOT unique and we need to look at the transfer
    # function and transfer type to determine what aggregate
    # this is, but we butcher based on version
    set dstagg ""

    set sql ""

    if {$V>=7.3} {
        set nsp [lindex [split $srcagg_ .] 0]
        set srcagg [lindex [split $srcagg_ .] 1]
        set srctyp [lindex [split $srcagg_ .] 2]
        set srcfnc [lindex [split $srcagg_ .] 3]
        set dstagg [lindex [split $dstagg_ .] 1]
        set sql "SELECT aggtransfn,
                        aggfinalfn,
                        aggtranstype::pg_catalog.regtype,
                        agginitval,
                        proargtypes\[0\]='pg_catalog.\"any\"'::pg_catalog.regtype AS anybasetype,
                        proargtypes\[0\]::pg_catalog.regtype AS fmtbasetype,
                        't'::boolean AS convertok
                   FROM pg_catalog.pg_aggregate a,
                        pg_catalog.pg_proc p,
                        pg_catalog.pg_namespace n,
                        pg_catalog.pg_type t
                  WHERE a.aggfnoid=p.oid
                    AND p.pronamespace=n.oid
                    AND n.nspname='$nsp'
                    AND p.proname='$srcagg'
                    AND a.aggtransfn='$srcfnc'::pg_catalog.regproc
                    AND a.aggtranstype=t.oid
                    AND t.typname='$srctyp'"
    } elseif {$V>7.1} {
        set dstagg [lindex [split $dstagg_ .] 0]
        set srcagg [lindex [split $srcagg_ .] 0]
        set srctyp [lindex [split $srcagg_ .] 1]
        set srcfnc [lindex [split $srcagg_ .] 2]
        set sql "SELECT aggtransfn,
                        aggfinalfn,
                        format_type(aggtranstype, NULL) AS aggtranstype,
                        agginitval,
                        aggbasetype=0 AS anybasetype,
                        CASE WHEN aggbasetype=0 THEN '-'
                             ELSE format_type(aggbasetype, NULL)
                        END AS fmtbasetype,
                        't'::boolean AS convertok
                   FROM pg_aggregate a,
                        pg_type t
                  WHERE a.aggname='$srcagg'
                    AND a.aggtransfn='$srcfnc'::regproc
                    AND a.aggtranstype=t.oid
                    AND t.typname='$srctyp'"
    } else {
        set dstagg [lindex [split $dstagg_ .] 0]
        set srcagg [lindex [split $srcagg_ .] 0]
        set srctyp [lindex [split $srcagg_ .] 1]
        set srcfnc [lindex [split $srcagg_ .] 2]
        set sql "SELECT aggtransfn1 AS aggtransfn,
                        aggfinalfn,
                        (SELECT typname
                           FROM pg_type
                          WHERE oid=a.aggtranstype1)
                        AS aggtranstype,
                        agginitval1 AS agginitval,
                        aggbasetype=0 AS anybasetype,
                        (SELECT typname
                           FROM pg_type
                          WHERE oid=a.aggbasetype)
                        AS fmtbasetype,
                        (aggtransfn2=0
                            AND aggtranstype2=0
                            AND agginitval2 IS NULL)
                        AS convertok
                   FROM pg_aggregate a,
                        pg_type t
                  WHERE a.aggname='$srcagg'
                    AND a.aggtransfn1='$srcfnc'::regproc
                    AND a.aggtranstype1=t.oid
                    AND t.typname='$srctyp'"
    }

    wpg_select $dbh_ $sql rec {
        # just in case dstagg got screwed up
        if {[string length $dstagg]==0} {
            set dstagg $dstagg_
        }
        set insql "CREATE AGGREGATE [::Database::quoteObject $dstagg] ("
        if {$rec(anybasetype)} {
            append insql "\n\tBASETYPE='any',"
        } else {
            append insql "\n\tBASETYPE=$rec(fmtbasetype),"
        }
        append insql "\n\tSFUNC=[::Database::quoteObject $rec(aggtransfn)],"
        append insql "\n\tSTYPE=$rec(aggtranstype),"
        if {[string length $rec(agginitval)]>0} {
            append insql \
            "\n\tINITCOND=[::Database::quoteObject $rec(agginitval)],"
        }
        if {$rec(aggfinalfn)!="-"} {
            append insql \
            "\n\tFINALFUNC=[::Database::quoteObject $rec(aggfinalfn)],"
        }
        set insql [string trimright $insql ","]
        append insql "\n);"
    }

    return $insql

}; # end proc ::Aggregates::clone


