# as_style.tcl --
#
#	This file implements package pga::style, which  ...
#
# Copyright (c) 2003 ActiveState Corporation, a division of Sophos
#
# Basic use:
#
# pga::style::init ?which?
# pga::style::reset ?which?
# pga::style::enable ?what ?args??
#	ie: enable control-mousewheel local|global
#

#package require NAME VERSION
package provide pga::style 1.1

namespace eval pga::style {
    #namespace export -clear *

    #variable highlightcolor "#ECE9D8"
    variable highlightbg "#316AC5" ; # SystemHighlight
    variable highlightfg "white"   ; # SystemHighlightText
    variable bg          "white"   ; # SystemWindow
    variable fg          "black"   ; # SystemWindowText
    variable tableselbg "#DDDDDD" ; # SystemHighlight
    variable tableselfg "#708090"   ; # SystemHighlightText
    variable tablebg          "white"   ; # SystemWindow
    variable tablefg          "black"   ; # SystemWindowText
    variable tablestripe      "#e0e8f0"   ; # SystemWindowText

    foreach C {tablebg tablefg tablestripe} {
        if {[info exists PgAcVar(pref,$C)]} {
           set $C $PgAcVar(pref,$C)
        }
    }

    # assume MouseWheel binding is the same across widget classes
    variable mw
    set mw(classes) [list Text Listbox Table TreeCtrl]
    set mw(binding) [bind Text <MouseWheel>]
    if {$::tcl_platform(platform) == "unix"} {
	set mw(binding4) [bind Text <4>]
	set mw(binding5) [bind Text <5>]
    }
}; # end of namespace pga::style

proc pga::style::init {args} {
    if {[llength $args]} {
	foreach what $args {
	    pga::style::init_$what
	}
    } else {
	foreach cmd [info procs init_*] {
	    $cmd
	}
    }

    if {$::tcl_platform(os) == "Windows CE"} {
	# WinCE is for small screens, with 240x320 (QVGA) the most common.
	# Adapt the defaults to that size.
	option add *font			{Tahoma 7} widgetDefault
	option add *Button.borderWidth		1 widgetDefault
	option add *Entry.borderWidth		1 widgetDefault
	option add *Listbox.borderWidth		1 widgetDefault
	option add *Spinbox.borderWidth		1 widgetDefault
	option add *Text.borderWidth		1 widgetDefault
	option add *Scrollbar.width		11 widgetDefault
	option add *padY			0 widgetDefault
    }
}
proc pga::style::reset {args} {
    if {[llength $args]} {
	foreach what $args {
	    pga::style::reset_$what
	}
    } else {
	foreach cmd [info commands pga::style::reset_*] {
	    $cmd
	}
    }
}
proc pga::style::enable {what args} {
    switch -exact $what {
	mousewheel { init_mousewheel }
	control-mousewheel {
	    set type [lindex $args 0]; # should be local or global
	    bind all <Control-MouseWheel> \
		[list ::pga::style::CtrlMouseWheel %W %D %X %Y $type]
	    bind all <Control-plus> \
		[list ::pga::style::CtrlMouseWheel %W 120 %X %Y $type]
	    bind all <Control-minus> \
		[list ::pga::style::CtrlMouseWheel %W -120 %X %Y $type]
	    if {$::tcl_platform(platform) == "unix"} {
		bind all <Control-ButtonPress-4> \
		    [list ::pga::style::CtrlMouseWheel %W 120 %X %Y $type]
		bind all <Control-ButtonPress-5> \
		    [list ::pga::style::CtrlMouseWheel %W -120 %X %Y $type]
	    }
	}
	default {
	    return -code error "unknown option \"$what\""
	}
    }
}
proc pga::style::disable {what args} {
    switch -exact $what {
	mousewheel { reset_mousewheel }
	control-mousewheel {
	    bind all <Control-MouseWheel> {}
	    bind all <Control-plus> {}
	    bind all <Control-minus> {}
	    if {$::tcl_platform(platform) == "unix"} {
		bind all <Control-ButtonPress-4> {}
		bind all <Control-ButtonPress-5> {}
	    }
	}
	default {
	    return -code error "unknown option \"$what\""
	}
    }
}

## Fonts
##
proc pga::style::init_fonts {args} {
    if {[lsearch -exact [font names] PGfont] != -1} return

    switch -exact $::tcl_platform(platform) {
	"unix" {
	    set size	-12
	    set family	Helvetica
	    set fsize	-12
	    set ffamily	Courier
	}
	"windows" {
	    set size	8
	    set family	Tahoma
	    set fsize	9
	    set ffamily	Courier
	}
	"mac" - "macintosh" {
	    set size	11
	    set family	"Lucida Grande"
	    set size	11
	    set family	Courier
	}
    }
    font create PGfont      -size $size -family $family
    font create PGfontBold  -size $size -family $family -weight bold
    font create PGfontFixed -size $fsize -family $ffamily
    for {set i -2} {$i <= 4} {incr i} {
	set isize  [expr {$size + ($i * (($size > 0) ? 1 : -1))}]
	set ifsize [expr {$fsize + ($i * (($fsize > 0) ? 1 : -1))}]
	font create PGfont$i      -size $isize -family $family
	font create PGfontBold$i  -size $isize -family $family -weight bold
	font create PGfontFixed$i -size $ifsize -family $ffamily
    }

    if {1 || $::tcl_platform(platform) == "unix"} {
	option add *Text.font		PGfontFixed widgetDefault
	option add *Button.font		PGfont widgetDefault
	option add *Canvas.font		PGfont widgetDefault
	option add *Checkbutton.font	PGfont widgetDefault
	option add *Entry.font		PGfont widgetDefault
	option add *Label.font		PGfont widgetDefault
	option add *Labelframe.font	PGfont widgetDefault
	option add *Listbox.font	PGfont widgetDefault
	option add *Menu.font		PGfont widgetDefault
	option add *Menubutton.font	PGfont widgetDefault
	option add *Message.font	PGfont widgetDefault
	option add *Radiobutton.font	PGfont widgetDefault
	option add *Spinbox.font	PGfont widgetDefault

	option add *Table.font		PGfont widgetDefault
	option add *TreeCtrl*font	PGfont widgetDefault
    }
}

proc pga::style::reset_fonts {args} {
}

proc pga::style::CtrlMouseWheel {W D X Y {what local}} {
    set w [winfo containing $X $Y]
    if {[winfo exists $w]} {
	set top [winfo toplevel $w]
	while {[catch {$w cget -font} font]
	       || ![string match "PGfont*" $font]} {
	    if {$w == $top} { return }
	    set w [winfo parent $w]
	}
	if {$what == "local"} {
	    # get current font size (0 by default) and adjust the current
	    # widget's font to the next sized preconfigured font
	    set cnt [regexp -nocase -- {([a-z]+)(\-?\d)?} $font -> name size]
	    if {$size == ""} {
		set size [expr {($D > 0) ? 1 : -1}]
	    } else {
		set size [expr {$size + (($D > 0) ? 1 : -1)}]
	    }
	    set font $name$size
	    if {[lsearch -exact [font names] $font] != -1} {
		catch {$w configure -font $font}
	    }
	} else {
	    # readjust all the font sizes based on the current one
	    set size [font configure PGfont -size]
	    incr size [expr {($D > 0) ? 1 : -1}]
	    # but we do have limits on how small/large things can get
	    if {$size < 6 || $size > 18} { return }
	    font configure PGfont      -size $size
	    font configure PGfontBold  -size $size
	    font configure PGfontFixed -size [expr {$size+1}]
	    # force reconfigure of this widget with the same font in
	    # case it doesn't have a WorldChanged function
	    catch {$w configure -font $font}
	    if {0} {
		# we shouldn't need this if the user isn't improperly
		# switching between global/local ctrl-mswhl modes
		for {set i -2} {$i <= 4} {incr i} {
		    font configure PGfont$i      \
			-size [expr {$size+$i}] -family $family
		    font configure PGfontBold$i  \
			-size [expr {$size+$i}] -family $family -weight bold
		    font configure PGfontFixed$i \
			-size [expr {$size+1+$i}] -family Courier
		}
	    }
	}
    }
}

## Misc
##
proc pga::style::init_misc {args} {
    option add *ScrolledWindow.ipad		0 widgetDefault
}

## Listbox
##
proc pga::style::init_listbox {args} {
    if {$::tcl_platform(platform) == "unix"} {
	variable highlightbg
	variable highlightfg
	variable bg
	variable fg
	option add *Listbox.background		$bg widgetDefault
	option add *Listbox.foreground		$fg widgetDefault
	option add *Listbox.selectBorderWidth	0 widgetDefault
	option add *Listbox.selectForeground	$highlightfg widgetDefault
	option add *Listbox.selectBackground	$highlightbg widgetDefault
    }
    option add *Listbox.activeStyle		dotbox widgetDefault
}

## Button
##
proc pga::style::init_button {args} {
    if {$::tcl_platform(platform) == "unix"} {
	option add *Button.padX			1 widgetDefault
	option add *Button.padY			2 widgetDefault
    }
}

## Entry
##
proc pga::style::init_entry {args} {
    if {$::tcl_platform(platform) == "unix"} {
	variable highlightbg
	variable highlightfg
	variable bg
	variable fg
	option add *Entry.background		$bg widgetDefault
	option add *Entry.foreground		$fg widgetDefault
	option add *Entry.selectBorderWidth	0 widgetDefault
	option add *Entry.selectForeground	$highlightfg widgetDefault
	option add *Entry.selectBackground	$highlightbg widgetDefault
	option add *Entry.disabledForeground	$fg widgetDefault
	option add *Entry.disabledBackground	$bg widgetDefault
    }
}

## Spinbox
##
proc pga::style::init_spinbox {args} {
    if {$::tcl_platform(platform) == "unix"} {
	variable highlightbg
	variable highlightfg
	variable bg
	variable fg
	option add *Spinbox.background		$bg widgetDefault
	option add *Spinbox.foreground		$fg widgetDefault
	option add *Spinbox.selectBorderWidth	0 widgetDefault
	option add *Spinbox.selectForeground	$highlightfg widgetDefault
	option add *Spinbox.selectBackground	$highlightbg widgetDefault
    }
}

## Text
##
proc pga::style::init_text {args} {
    if {$::tcl_platform(platform) == "unix"} {
	variable highlightbg
	variable highlightfg
	variable bg
	variable fg
	option add *Text.background		$bg widgetDefault
	option add *Text.foreground		$fg widgetDefault
	option add *Text.selectBorderWidth	0 widgetDefault
	option add *Text.selectForeground	$highlightfg widgetDefault
	option add *Text.selectBackground	$highlightbg widgetDefault
    }
}

## Menu
##
proc pga::style::init_menu {args} {
    if {$::tcl_platform(platform) == "unix"} {
	variable highlightbg
	variable highlightfg
	option add *Menu.activeBackground	$highlightbg widgetDefault
	option add *Menu.activeForeground	$highlightfg widgetDefault
	option add *Menu.activeBorderWidth	0 widgetDefault
	option add *Menu.highlightThickness	0 widgetDefault
	option add *Menu.borderWidth		1 widgetDefault
    }
    option add *Menu.tearOff		0 widgetDefault
}

## Menubutton
##
proc pga::style::init_menubutton {args} {
    variable highlightbg
    variable highlightfg
    option add *Menubutton.activeBackground	$highlightbg widgetDefault
    option add *Menubutton.activeForeground	$highlightfg widgetDefault
    option add *Menubutton.activeBorderWidth	0 widgetDefault
    option add *Menubutton.highlightThickness	0 widgetDefault
    option add *Menubutton.borderWidth		0 widgetDefault
    option add *Menubutton*padX			4 widgetDefault
    option add *Menubutton*padY			2 widgetDefault
}

## Scrollbar
##
proc pga::style::init_scrollbar {args} {
    if {$::tcl_platform(platform) == "unix"} {
	option add *Scrollbar.width		12 widgetDefault
	option add *Scrollbar.troughColor	#bdb6ad widgetDefault
    }
    option add *Scrollbar.borderWidth		1 widgetDefault
    option add *Scrollbar.highlightThickness	0 widgetDefault
}

## PanedWindow
##
proc pga::style::init_panedwindow {args} {
    option add *Panedwindow.borderWidth		0 widgetDefault
    option add *Panedwindow.sashwidth		3 widgetDefault
    option add *Panedwindow.showhandle		0 widgetDefault
    option add *Panedwindow.sashpad		0 widgetDefault
    option add *Panedwindow.sashrelief		flat widgetDefault
    option add *Panedwindow.relief		flat widgetDefault
}


## Tree widget from BWidgets
##
proc pga::style::init_tree {args} {

    variable bg

    option add *Tree*background         $bg startupFile

}; # end pga::style::init_tree

## ComboBox widget from BWidgets
##
proc pga::style::init_combobox {args} {

    variable bg

    option add *ComboBox*background         $bg startupFile
    option add *ComboBox*borderWidth         1 startupFile

}; # end pga::style::init_combobox


## SpinBox widget from BWidgets
##
proc pga::style::init_spinbox {args} {

    variable bg

    option add *SpinBox*background         $bg startupFile
    option add *SpinBox*borderWidth         1 startupFile

}; # end pga::style::init_spinbox



## Tablelist widget from Csaba Nemeth
##
proc pga::style::init_tablelist {args} {

    variable bg
    variable fg
    variable tableselbg
    variable tableselfg
    variable tablebg
    variable tablefg
    variable tablestripe

    option add *Tablelist.background         $tablebg widgetDefault
    option add *Tablelist.foreground         $tablefg widgetDefault
    option add *Tablelist.stripeBackground   $tablestripe widgetDefault

}; #end pga::style::init_tablelist

## Table widget from Jeff Hobbs
##
proc pga::style::init_table {args} {

    variable bg
    variable fg
    variable highlightbg
    variable highlightfg
    variable tablebg
    variable tablefg

    option add *Table.background         $tablebg widgetDefault
    option add *Table.foreground         $tablefg widgetDefault
    option add *Table.borderWidth        1 widgetDefault
    option add *Table.highlightThickness 0 widgetDefault
    option add *Table.padX               2 widgetDefault
    option add *Table.padY               2 widgetDefault
    option add *Table.selectType         row widgetDefault

    return

}; #end pga::style::init_tablelist

## MouseWheel
##
proc pga::style::MouseWheel {wFired D X Y} {
    if {[bind [winfo class $wFired] <MouseWheel>] == ""} {
	set w [winfo containing $X $Y]
	# if we are outside the app, try and scroll the focus widget
	if {![winfo exists $w]} { catch {set w [focus]} }
	if {[winfo exists $w]} {
	    # scrollbars have different call conventions
	    if {[winfo class $w] == "Scrollbar"} {
		catch {tk::ScrollByUnits $w \
			   [string index [$w cget -orient] 0] \
			   [expr {-($D/30)}]}
	    } else {
		catch {$w yview scroll [expr {- ($D / 120) * 4}] units}
	    }
	}
    }
}
proc pga::style::init_mousewheel {args} {
    variable mw

    # Create a catch-all MouseWheel proc & binding and
    # alter default bindings to allow toplevel binding to control all
    bind all <MouseWheel> [list ::pga::style::MouseWheel %W %D %X %Y]
    foreach class $mw(classes) {
	bind $class <MouseWheel> {}
    }
    #if {[bind [winfo toplevel %W] <MouseWheel>] ne ""} { continue }
    #%W yview scroll [expr {- (%D / 120) * 4}] units

    if {$::tcl_platform(platform) == "unix"} {
	# Support for mousewheels on Linux/Unix commonly comes through
	# mapping the wheel to the extended buttons.
	bind all <4> [list ::pga::style::MouseWheel %W 120 %X %Y]
	bind all <5> [list ::pga::style::MouseWheel %W -120 %X %Y]
	foreach class $mw(classes) {
	    bind $class <4> {}
	    bind $class <5> {}
	}
    }
}
proc pga::style::reset_mousewheel {args} {
    # Remove catch-all MouseWheel binding and restore default bindings
    variable mw

    bind all <MouseWheel> {}
    foreach class $mw(classes) {
	bind $class <MouseWheel> $mw(binding)
    }
    if {$::tcl_platform(platform) == "unix"} {
	bind all <4> {}
	bind all <5> {}
	foreach class $mw(classes) {
	    bind $class <4> $mw(binding4)
	    bind $class <5> $mw(binding5)
	}
    }
}

package provide pga::style 1.1
