# ----------------- LICENSE ----------------------------------------------------
# see http://opensource.org/licenses/bsd-license.html
#
# Copyright (c) 2004, PGACCESS-TEAM
# All rights reserved.
#
# Redistribution and use in source and binary forms,
# with or without modification, are
# permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the
#      above copyright notice, this list of
#      conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce
#      the above copyright notice, this list of
#      conditions and the following disclaimer in the
#      documentation and/or other materials
#      provided with the distribution.
#    * Neither the name of the PGACCESS-TEAM nor the
#      names of its contributors may
#      be used to endorse or promote products derived
#      from this software without specific
#      prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# ----------------- END LICENSE ------------------------------------------------
#
# Contibutor(s)
#       Robert Strong (trebor_gnorts at hotmail dot com)
#       Chris Maj (cmaj at freedomcorpse dot com)
#

#----------------------------------------------------------
# VisualQB
#
#   provides for rapid query building
#
#----------------------------------------------------------

# The following is the variable for Tktable (which must be global)
global tbquery

namespace eval VisualQB {
    variable Win
    # Array containing grid row numbers. e.g. 'set gridRow(Field) 1'
    variable gridRow
    # This array will hold all the local variables
    # Currently:
    #   lbnam   Name of selected database table
    #   hlite   Type of selected object, can be:
    #           link, tabl, grid, none(default)
    #   ...and many, many more
    variable vqb
    set vqb(hlite) none
}


#----------------------------------------------------------
# ::VisualQB::init
#----------------------------------------------------------
#
proc ::VisualQB::init {} {

    global PgAcVar tbquery cbw
    variable vqb
    variable gridRow
    variable Win

    # cbw = checkbox window number in grid
    # each window has to have a unique name
    # numbers 1 to 10 have already been created in the grid
    # this var is incremented each time a column is inserted into the grid
    set cbw 10

    catch {unset vqb}
    set vqb(rescriteria) {}
    set vqb(links) {}
    set vqb(ntables) 0
    set vqb(newtablename) {}

    # Array of lists containing table name and table id
    # (needed for dragging individual tables)
    set vqb(tid) {}

    # set row numbers in grid
    set gridRow(Field)    1
    set gridRow(Alias)    2
    set gridRow(Table)    3
    set gridRow(Function) 4
    set gridRow(Sort)     5
    set gridRow(Visible)  6
    set gridRow(Criteria) 7
    set gridRow(Or)       8

    # Hide 'Function' row by setting height to zero
    #$Win(tbl) height $gridRow(Function) 0

    # Set checkbuttons to off
    for {set c 1} {$c < [$Win(tbl) cget -columns]} {incr c} {
        set wn [$Win(tbl) window cget $gridRow(Visible),$c -window]
        $wn deselect
    }

    # Set row headings for query grid
    array unset tbquery
    #$Win(tbl) set $gridRow(Field),0 "Field: "
    #$Win(tbl) set $gridRow(Alias),0 "Alias: "
    #$Win(tbl) set $gridRow(Table),0 "Table: "
    #$Win(tbl) set $gridRow(Function),0 "Function: "
    #$Win(tbl) set $gridRow(Sort),0 "Sort: "
    #$Win(tbl) set $gridRow(Visible),0 "Visible: "
    #$Win(tbl) set $gridRow(Criteria),0 "Criteria: "
    #$Win(tbl) set $gridRow(Or),0 "Or: "

    set cols [list "Field" "Alias" "Table" "Function" "Sort" "Visible" "Criteria" "Or"]
    foreach c $cols {
        lappend wincols 0 $c left
    }
    $Win(tbl) configure -columns $wincols

    # give all columns a name in case they move around
    # (although their title will always be correct)
    set i -1
    foreach c $cols {
        $Win(tbl) columnconfigure [incr i] \
            -name col$c
    }

    $Win(tbl) columnconfigure colAlias \
        -editable 1

    $Win(tbl) columnconfigure colFunction \
        -editable 1

    $Win(tbl) columnconfigure colSort \
        -editable 1 \
        -editwindow ComboBox

    $Win(tbl) columnconfigure colVisible \
        -editable 1 \
        -editwindow checkbutton

    # Set up default query properties
    set PgAcVar(properties,DISTINCT) 0;           # Yes = 1, No = 0
    set PgAcVar(properties,ON) {};                # If DISTINCT = yes, this is added to the sql command
    set PgAcVar(properties,FROM) 1;               # New 'join' syntax = 1, Old 'where' syntax = 0
    set PgAcVar(properties,CRITERIA) 1;           # Process criteria by row = 1, Process by column = 0
    set PgAcVar(properties,LIMIT) {};             # Integer for limit
    set PgAcVar(properties,OFFSET) {};            # If limit > 0, then offset is added to sql
    set PgAcVar(properties,ShowFunction) 0;       # Yes = 1, No = 0 (This switches on aggregation, GROUP BY & HAVING)

    # Set up default join type
    set PgAcVar(properties,JoinType) 0;           # INNER = 0, LEFT OUTER = 1, RIGHT OUTER = 2, FULL OUTER = 3

}; # end proc ::VisualQB::init


#----------------------------------------------------------
# ::VisualQB::print
#----------------------------------------------------------
#
proc ::VisualQB::print {} {

    variable Win
    variable vqb

    #set g [string trimleft [wm geometry .pgaw:VisualQuery] "="]
    #set vqb(xsize) [lindex [split $g "x+"] 0]
    #set vqb(ysize) [lindex [split $g "x+"] 1]

    #::Printer::init "::VisualQB::printcallback"

    $Win(cv) postscript -file "test.ps" -pagex 0.i -pagey 11.i -pageanchor nw
}; # end proc ::VisualQB::print


#----------------------------------------------------------
# ::VisualQB::printcallback
#----------------------------------------------------------
#
proc ::VisualQB::printcallback {fid} {
    variable Win
    variable vqb

    ::Printer::printStart $fid $vqb(xsize) $vqb(ysize) 1
    ::Printer::printPage $fid 1 $Win(cv)
    ::Printer::printStop $fid

}; # end proc ::VisualQB::print


#----------------------------------------------------------
# ::VisualQB::createDropDown
#----------------------------------------------------------
#
proc ::VisualQB::createDropDown {} {

    global PgAcVar
    variable vqb
    variable Win

    if { [winfo exists .pgaw:VisualQuery.ddf] } {
        destroy .pgaw:VisualQuery.ddf
        return
    }

    create_drop_down .pgaw:VisualQuery 70 27 200
    focus .pgaw:VisualQuery.ddf.sb

    foreach tbl [Database::getTablesList] {
        .pgaw:VisualQuery.ddf.lb insert end $Win(tbl)
    }

    bind .pgaw:VisualQuery.ddf.lb <ButtonRelease-1> {
        set i [.pgaw:VisualQuery.ddf.lb curselection]
        if { $i != "" } {
            set VisualQB::vqb(newtablename) \
                [.pgaw:VisualQuery.ddf.lb get $i]
            VisualQB::addNewTable
        }
        destroy .pgaw:VisualQuery.ddf
        break
    }

    return

}; # end proc ::VisualQB::createDropDown


#----------------------------------------------------------
# ::VisualQB::addNewTable
#----------------------------------------------------------
#
proc ::VisualQB::addNewTable {{tabx 0} {taby 0} {alias -1}} {

    global PgAcVar CurrentDB
    variable vqb
    variable Win

    if {$vqb(newtablename)==""} return
    set fldlist {}
    set l [split [string map {\" \'} $vqb(newtablename)] "."]
    if {[llength $l] == 2} {
    set schemaname [lindex $l 0]
    }
    set tablename [lindex $l end]
    set ver [string range [::Database::getPgVersion $CurrentDB] 0 2]
    setCursor CLOCK
    set fldlist [::Database::getColumnsList $vqb(newtablename)]
    setCursor DEFAULT
    if {$fldlist==""} {
    showError [format [intlmsg "Table '%s' not found!"] $vqb(newtablename)]
    return
    }
    if {$alias==-1} {
    set tabnum $vqb(ntables)
    } else {
    regsub t $alias "" tabnum
    }
    set vqb(tablename$tabnum) $vqb(newtablename)
    set vqb(tablestruct$tabnum) $fldlist
    set vqb(tablealias$tabnum) "t$tabnum"
    set vqb(ali_t$tabnum) $vqb(newtablename)
    set vqb(tablex$tabnum) $tabx
    set vqb(tabley$tabnum) $taby
    incr vqb(ntables)
    if {$vqb(ntables)==1} {
        repaintAll
    } else {
        drawTable [expr $vqb(ntables)-1]
    }
    set vqb(newtablename) {}
    focus $Win(entertable)

}; # end proc ::VisualQB::addNewTable


#----------------------------------------------------------
# ::VisualQB::drawTable
#----------------------------------------------------------
#
proc ::VisualQB::drawTable {it} {

    global PgAcVar
    variable vqb
    variable Win

    set cv $Win(cv)

    # Get the number of pixels for 29.7cm and 21cm on the canvas
    set nx [winfo fpixels $cv 29.7c]
    set ny [winfo fpixels $cv 21.0c]
    if { $vqb(tablex$it) == 0 } {
        set posy 10
        set allbox [$cv bbox tFrame]
        if { $allbox == "" } {
            set posx 10
        } else {
            set posx [expr 20 + [lindex $allbox 2]]
        }
        # ToDo: Try to get smarter placement
        if {$posx > $nx} {
            set posx 20
            set posy 20
        }
        set vqb(tablex$it) $posx
        set vqb(tabley$it) $posy
    } else {
        set posx [expr int($vqb(tablex$it))]
        set posy [expr int($vqb(tabley$it))]
    }

    # Remove leading and trailing double-quotes
    set tablename [string trim $vqb(tablename$it) \"]
    set tablealias $vqb(tablealias$it)

    # Create table window (frame, label, Listbox and scrollbar)
    set fnam tab$it
    frame $cv.$fnam \
        -borderwidth 3 \
        -cursor bottom_right_corner \
        -relief raised
    label $cv.$fnam.lab \
        -borderwidth 2 \
        -cursor left_ptr \
        -text $tablename
    ListBox $cv.$fnam.lb \
        -background #fefefe \
        -borderwidth 0 \
        -cursor left_ptr \
        -deltay 14 \
        -dragenabled 1 \
        -dragevent 1 \
        -dropcmd ::VisualQB::lbdrop \
        -dropenabled 1 \
        -dropovermode i \
        -foreground #000000 \
        -highlightthickness 0 \
        -relief flat \
        -selectfill 1 \
        -selectforeground white -selectbackground darkblue \
        -selectmode single \
        -yscrollcommand [subst {$cv.$fnam.sb set}]
    scrollbar $cv.$fnam.sb \
        -borderwidth 1 \
        -command [list VisualQB::Scroll_Change $cv.$fnam.lb] \
        -cursor left_ptr \
        -highlightthickness 0 \
        -orient vert

    # Get the width of the longest field and height of fields
    set w 0
    set numflds 0
    foreach fld $vqb(tablestruct$it) {
    set tmp [font measure $PgAcVar(pref,font_normal) -displayof $cv.$fnam.lb $fld]
    if {$tmp > $w} {
      set w $tmp}
    incr numflds
    }

    # Add one to number of fields to allow for star(*) entry
    set h [expr [expr $numflds + 1] * 14]

    pack $cv.$fnam.lab -side top -fill x
    pack $cv.$fnam.sb -side right -fill y
    pack $cv.$fnam.lb -side left -expand 1 -fill both

    # Add star (*) to list of fields
    $cv.$fnam.lb insert end * -text * -data $it
    # Add rest of fields
    foreach fld $vqb(tablestruct$it) {
    $cv.$fnam.lb insert end $fld -text $fld -data $it
    }

    # Add 30 to width to allow for left border where an image can be displayed
    # Add 5 to width to allow for 3D border
    # Add 14 to width for scroll bar
    # Add 30 to height to allow for the label
    # Add 5 to height to allow for 3D border
    set tid [$cv create window $posx $posy -anchor nw -window $cv.$fnam \
       -height [expr $h + 35] -width [expr $w + 49] -tags [subst {tFrame $tablename}]]
    lappend vqb(tid) [list $cv.$fnam $tid]

    $cv.$fnam.lb bindText <Button-1> [list VisualQB::clickListbox $cv.$fnam.lb]
    #bind $cv.$fnam.lb <<ListboxSelect>> VisualQB::clickListbox
    $cv.$fnam.lb bindText <Double-Button-1> [list VisualQB::dblclickListbox $cv.$fnam.lb]

    # Set up events for dragging the tables on the canvas
    # ---------------------------------------------------
    bind $cv.$fnam.lab <ButtonPress-1> {
    set dragx %x
    set dragy %y
    set w [winfo parent %W]
    set tid [VisualQB::gettid $w]
    %W configure -cursor hand1
    }

    bind $cv.$fnam.lab <B1-Motion> {
    set dx [expr %x - $dragx]
    set dy [expr %y - $dragy]
    .pgaw:VisualQuery.pw.f0.frame.c move $tid $dx $dy
    VisualQB::drawLinks
    update idletasks;                                    # Helps reduce flicker
    }

    bind $cv.$fnam.lab <ButtonRelease-1> {
    %W configure -cursor left_ptr
    }

    #
    # THIS IS COOL !!!
    #
    # Set up events for resizing the tables on the canvas
    # ---------------------------------------------------
    # Change the cursor shape in frame
    #    1      2      3
    #    ---------------
    #    |              |
    #  4 |      5       | 6
    #    |              |
    #    ---------------
    #    7      8      9
    bind $cv.$fnam <Motion> {
    set p [VisualQB::Cursor_Position %W %x %y]
    switch $p {
      1 {%W configure -cursor top_left_corner}
      2 {%W configure -cursor top_side}
      3 {%W configure -cursor top_right_corner}
      4 {%W configure -cursor left_side}
      5 {%W configure -cursor left_ptr}
      6 {%W configure -cursor right_side}
      7 {%W configure -cursor bottom_left_corner}
      8 {%W configure -cursor bottom_side}
      9 {%W configure -cursor bottom_right_corner}
      default {%W configure -cursor bottom_right_corner}
    }
    }

    bind $cv.$fnam <ButtonPress-1> {
    set resizex %x
    set resizey %y
    set cv  .pgaw:VisualQuery.pw.f0.frame.c
    set tid [VisualQB::gettid %W]
    }

    bind $cv.$fnam <B1-Motion> {
    set dx [expr %x - $resizex]
    set wid [$cv itemcget $tid -width]
    switch $p {
      1 -
      4 -
      7 {
       set newWidth [expr $wid - $dx]
       if {$newWidth >= 100} then {
         $cv itemconfig $tid -width $newWidth
         $cv move $tid $dx 0
       }
    }
      3 -
      6 -
      9 {
       set newWidth [expr $wid + $dx]
       if {$newWidth >= 100} then {
         $cv itemconfig $tid -width $newWidth
         set resizex %x
       }
    } 
    }
    set dy [expr %y - $resizey]
    set ht [$cv itemcget $tid -height]
    switch $p {
      1 -
      2 -
      3 {
       set newHeight [expr $ht - $dy]
       if {$newHeight >= 60} then {
         $cv itemconfig $tid -height $newHeight
         $cv move $tid 0 $dy
       }
    }
      7 -
      8 -
      9 {
       set newHeight [expr $ht + $dy]
       if {$newHeight >= 60} then {
         $cv itemconfig $tid -height $newHeight
         set resizey %y
       }
    } 
    }
    VisualQB::drawLinks
    update idletasks;                                   # Helps reduce flicker
    }

    bind $cv.$fnam <ButtonRelease-1> {
    #%W configure -cursor left_ptr
    }

}; # end proc ::VisualQB::drawTable


#----------------------------------------------------------
# ::VisualQB::Cursor_Position
#----------------------------------------------------------
#
# The following numbers indicate position in frame
#    1      2      3
#    ---------------
#    |              |
#  4 |      5       | 6
#    |              |
#    ---------------
#    7      8      9
proc ::VisualQB::Cursor_Position {w x y} {
    set cornerSize 10
    set result 9

    # Get X position  
    if {$x < $cornerSize} {
    set result 1
    } elseif {$x > [expr [winfo width $w] - $cornerSize]} {
    set result 3
    } else {
    set result 2
    }

    # Get Y position
    if {$y > $cornerSize} {
    if {$y < [expr [winfo height $w] - $cornerSize]} {
      incr result 3
    } else {
      incr result 6
    }
    }
    return $result

}; # end proc ::VisualQB::Cursor_Position


#----------------------------------------------------------
# ::VisualQB::Scroll_Change
#----------------------------------------------------------
#
proc ::VisualQB::Scroll_Change {listbox scroll num units} {
    $listbox yview scroll $num $units
    update
    VisualQB::drawLinks
}; # end proc Scroll_Change


#----------------------------------------------------------
# ::VisualQB::showFunctions
#----------------------------------------------------------
#
proc ::VisualQB::showFunctions {} {
    variable gridRow
    variable Win

    set h [$Win(tbl) height $gridRow(Function)]

    if {$h != 0} then {
        $Win(tbl) height $gridRow(Function) 0
    } else {
        $Win(tbl) height $gridRow(Function) 1
    }

}; # end proc showFunctions



#----------------------------------------------------------
# ::VisualQB::startFieldEdit --
#
#   fired when beginning interactive tablelist editing
#
# Parameters:
#   win_    name of the tablelist widget
#   row_    field row to edit
#   col_    field attribute column to edit
#   txt_    current data in the row/column chosen for edit
#
# Returns:
#   the text to place in the editing widget (usually an
#   entry box)
#
# Modfies:
#   text or other displayed information in the widget
#----------------------------------------------------------
#
proc ::VisualQB::startFieldEdit {win_ row_ col_ txt_} {
    variable Win

    set wn [winfo parent $win_]

    set entrypath [$Win(tbl) entrypath]
    set editwinpath [$Win(tbl) editwinpath]
    set colname [$Win(tbl) columncget $col_ -name]

    # check for columns that we use spinboxes, comboxes, etc.
    if { $colname == "colSort" } {
        $editwinpath configure \
            -editable 0 \
            -values [list Unsorted Ascending Descending]
    # might not have a path, ie. it's a boolean checkbutton;
    # otherwise, its a plain old entry widget
    } elseif { $entrypath != "" } {
        # setup a binding on the automagic entry for double clicks
        # to open a large window for multiline editing
        $entrypath configure -textvariable vqb(moretext)
        bind $entrypath <Double-1> {
            moreText [%W get] 1 [%W cget -textvariable]
        }
    }

    return $txt_

}; # end proc ::VisualQB::startFieldEdit


#----------------------------------------------------------
# ::VisualQB::clickListbox --
#
#   highlight the name of a column in a graphical table
#   representation
#----------------------------------------------------------
#
proc ::VisualQB::clickListbox {w i} {
    variable vqb
    VisualQB::deselectGrid
    VisualQB::deselectLinks
    VisualQB::deselectTables
    $w selection set $i
    set vqb(hlite) tabl
    set vqb(lbnam) $w
}; # end proc ::VisualQB::clickListbox


#----------------------------------------------------------
# ::VisualQB::dblclickListbox --
#
#   adds a column to be included in the SQL statement
#----------------------------------------------------------
#
proc ::VisualQB::dblclickListbox {w i} {

    global PgAcVar cbw
    variable vqb
    variable gridRow
    variable Win

    VisualQB::deselectGrid
    VisualQB::deselectLinks
    VisualQB::deselectTables
    $w selection set $i
    set vqb(hlite) tabl
    set vqb(lbnam) $w

    set f [winfo parent $w]
    # TODO put these table frames in Win array
    set tablename [$f.lab cget -text]

    $Win(tbl) insert end [list $i {} $tablename {} "Unsorted" 1]
    $Win(tbl) cellselection set end,0 end,end
    $Win(tbl) seecell end,0

}; # end proc ::VisualQB::dblclickListbox 


#----------------------------------------------------------
# ::VisualQB::gettid
#
#   Get qbTable ID. This is the ID the canvas uses, and is
#   saved in vqb(tid) when the table is created.
#   {see proc ::VisualQB::drawTable}
#----------------------------------------------------------
#
proc ::VisualQB::gettid {w} {
    variable vqb

    foreach lst $vqb(tid) {
    if {[lindex $lst 0] == $w} {
      set tid [lindex $lst 1]
      break;
    }
    }
    return $tid

}; # end proc ::VisualQB::gettid


#--------------------------------------------------------------------
# ::VisualQB::lbdrop
#
#   Add a new link to query when a field is dropped onto
#   a field in another table. Links to the star (*) field
#   are not allowed. Also links to other fields in the same
#   table are not allowed. (This may be changed in a future
#   release).
#
# Arguments:
#   target: pathname of the listbox dropped onto
#   source: pathname of the drag source
#   lst   : list describing where the drop occurs. It can be
#           {widget}
#           {item item} or
#           {position index}
#   op    : current operation
#   type  : data type (of drag source, should be LISTBOX_ITEM)
#   data  : data (of drag source)
#
# Returns:
#   none
#--------------------------------------------------------------------
#
proc ::VisualQB::lbdrop {target source lst op type data} {
    global PgAcVar
    variable vqb

    # 'source' is the name of the listbox canvas, therefore two calls
    # to 'winfo' are required to get the tcl name of the table (the
    # first call returns the name of the listbox, the second returns
    # name of the table).
    set sourcetable [winfo parent [winfo parent $source]]
    set targetfield [lindex $lst end]
    set targettable [winfo parent $target]

    # Disallow links to * field
    if {$data == "*" || $targetfield == "*"} return;

    # vqb(links) holds sourcetable, sourcefield, targettable, targetfield and jointype
    # Default join type is INNER JOIN. Can be one of INNER, LEFT, RIGHT, FULL
    if {$sourcetable != $targettable} {
    lappend vqb(links) [list $sourcetable $data $targettable $targetfield "INNER"]
    drawLinks
    }
}; # end proc ::VisualQB::lbdrop


#----------------------------------------------------------
# ::VisualQB::tableDrop
#----------------------------------------------------------
#
proc ::VisualQB::tableDrop {target source x y currentop type data} {
    variable Win
    variable vqb

    set it [[winfo parent $source] itemcget $data -data]
    set tablename [string trim $vqb(tablename$it) \"]

    $Win(tbl) insert end [list $data {} $tablename {} "Unsorted" 1]
    $Win(tbl) cellselection set end,0 end,end
    $Win(tbl) seecell end,0

}; # end proc ::VisualQB::tableDrop


#----------------------------------------------------------
# ::VisualQB::drawLinks
#----------------------------------------------------------
#
proc ::VisualQB::drawLinks {} {

    global PgAcVar
    variable vqb
    variable Win

    if {[llength $vqb(links)] == 0} return

    $Win(cv) delete links
    set i 0

    # vqb(links) is an array of lists
    # each list (link) contains sourcetable, sourcefield, targettable, targetfield, jointype
    foreach link $vqb(links) {
    set sourcetable [lindex $link 0]
    set sourcefield [lindex $link 1]
    set targettable [lindex $link 2]
    set targetfield [lindex $link 3]

    # Compute the source and destination right edge
    set tid [gettid $sourcetable]
    set scoords [$Win(cv) bbox $tid];          # source table coords
    set sre [lindex $scoords 2];          # source table right edge coord
    set tid [gettid $targettable]
    set tcoords [$Win(cv) bbox $tid];          # target table coords
    set dre [lindex $tcoords 2];          # destination table right edge coord

    # Compute field bound boxes
    set slb $sourcetable.lb
    set sidx [$slb index $sourcefield];   # source index
    set tlb $targettable.lb
    set tidx [$tlb index $targetfield];   # target index

    # Compute the auxiliary lines
    if {$sre < $dre} {
      # Source object is on the left of target object
      set x1 $sre
      set y1 [linkYcoord $sourcetable $sourcefield $scoords]      
      $Win(cv) create line $x1 $y1 [expr $x1 + 10] $y1 -tags [subst {links lkid$i}] -width 3
      set x2 [lindex $tcoords 0]
      #set y2 [expr $tidx * 14 + [lindex $tcoords 1] + 30]
      set y2 [linkYcoord $targettable $targetfield $tcoords]      
      $Win(cv) create line [expr $x2 - 10] $y2 $x2 $y2 -tags [subst {links lkid$i}] -width 3
      $Win(cv) create line [expr $x1 + 10] $y1 [expr $x2 - 10] $y2 -tags [subst {links lkid$i}] -width 2
    } else {
      # source object is on the right of target object
      set x1 $dre
      set y1 [linkYcoord $targettable $targetfield $tcoords]
      $Win(cv) create line $x1 $y1 [expr $x1 + 10] $y1 -tags [subst {links lkid$i}] -width 3
      set x2 [lindex $scoords 0]
      #set y2 [expr $sidx * 14 + [lindex $scoords 1] + 30]
      set y2 [linkYcoord $sourcetable $sourcefield $scoords]      
      $Win(cv) create line $x2 $y2 [expr $x2 - 10] $y2 -tags [subst {links lkid$i}] -width 3
      $Win(cv) create line [expr $x1 + 10] $y1 [expr $x2 - 10] $y2 -tags [subst {links lkid$i}] -width 2
      }
      incr i
    }

    $Win(cv) lower links

}; # end proc ::VisualQB::drawLinks


#----------------------------------------------------------
# ::VisualQB::linkYcoord
#
#   Calculate the y coordinate for given table and field
#----------------------------------------------------------
#
proc ::VisualQB::linkYcoord {table field coords} {
    set lstbox $table.lb
    set idx [$lstbox index $field];                                 # index of field in table
    set lineheight [$lstbox cget -deltay]
    set nelements [llength [$lstbox items]]
    set wh [expr ($nelements + 1) * $lineheight];                   # height of list box
    set sc [$table.sb get];                                         # get top and bottom fractions of scrollbox
    set sctop [lindex $sc 0]
    set scbottom [lindex $sc 1]
    set topIndex [expr round([expr ($sctop * $wh) / $lineheight])]
    set bottomIndex [expr round([expr ($scbottom * $wh) / $lineheight])]
    if {$idx < $topIndex} {
    set y [expr [lindex $coords 1] + 15]
    } elseif {$idx >= $bottomIndex} {
    set y [expr [lindex $coords 3] - 1]
    } else {
    set y [expr ($idx - $topIndex) * $lineheight]
    set y [expr $y + [lindex $coords 1] + [winfo y $lstbox] + 8]
    }
    return $y
};   # end proc ::VisualQB::linkYcoord


#----------------------------------------------------------
# ::VisualQB::linkClick
#----------------------------------------------------------
#
proc ::VisualQB::linkClick {obj} {

    global PgAcVar
    variable vqb
    variable Win

    set taglist [$Win(cv) gettags $obj]

    # deselct everything
    deselectGrid
    deselectLinks
    deselectTables

    set lt [lindex $taglist 1]
    set vqb(CurrentLink) [string range $lt 4 end]
    $Win(cv) addtag hili withtag $lt
    foreach i [$Win(cv) find withtag hili] {
        $Win(cv) itemconfigure $i -fill blue
    }
    set vqb(hlite) link

}; # end proc ::VisualQB::linkClick


#----------------------------------------------------------
# ::VisualQB::canvasClick
#----------------------------------------------------------
#
proc ::VisualQB::canvasClick {x y} {

    global PgAcVar
    variable vqb
    variable Win

    # As the scrollregion for the canvas is larger than the
    # screen, we need to convert the screen x,y coordinates
    # to the canvas x,y coordinates.
    set cx [$Win(cv) canvasx $x] 
    set cy [$Win(cv) canvasy $y]
    set obj [$Win(cv) find overlapping $cx $cy $cx $cy]
    if {$obj != ""} {
        linkClick $obj
    } else {
        deselectGrid
        deselectLinks
        deselectTables
        set vqb(hlite) none
    }
}; # end proc ::VisualQB::canvasClick


#----------------------------------------------------------
# ::VisualQB::canvasButton3
# 
#   show popup menu on right click a link
#----------------------------------------------------------
#
proc ::VisualQB::canvasButton3 {w x y} {

    global PgAcVar
    variable vqb
    variable Win

    # As the scrollregion for the canvas is larger than the
    # screen, we need to convert the screen x,y coordinates
    # to the canvas x,y coordinates.
    set cx [$Win(cv) canvasx $x] 
    set cy [$Win(cv) canvasy $y]
    set obj [$Win(cv) find overlapping $cx $cy $cx $cy]
    if {$obj != ""} {
        linkClick $obj
        tk_popup $Win(cv).pop [winfo pointerx $w] [winfo pointery $w]    
    }

}; # end proc ::VisualQB::canvasButton3 



#----------------------------------------------------------
# ::VisualQB::computeSQL
#----------------------------------------------------------
#
proc ::VisualQB::computeSQL {} {

    global PgAcVar
    variable vqb

    # Allow for different types of queries
    # call appropriate routine (SELECT is currently the only type)
    set sqlcmd [computeSELECT]
    return [set vqb(qcmd) $sqlcmd]

}; # end proc ::VisualQB::computeSQL

#----------------------------------------------------------
# ::VisualQB::computeSELECT
#----------------------------------------------------------
#
proc ::VisualQB::computeSELECT {} {

    global PgAcVar
    variable vqb
    variable gridRow
    variable Win

    set vqb(rescriteria) {}

    if {$PgAcVar(properties,DISTINCT) != 0} {
    set don $PgAcVar(properties,ON)
    if {$don != ""} {
      set sqlcmd "SELECT DISTINCT ON ($don) "
    } else {
      set sqlcmd "SELECT DISTINCT "
    }
    } else {
    set sqlcmd "SELECT "
    }
    set sqlstart 0

    # Add fields to the select command
    # First check if 'Function' row is displayed
    # If yes, check if 'Function' row contains a function (avg(), count(),
    # max(), min(), stddev(), sum() or variance() )
    # If yes surround field name with function
    # If field starts with an equals sign (=) then assume that text in the
    # field cell is an expression, so just add to sqlcmd

    set h [$Win(tbl) height $gridRow(Function)]
    for {set c 1} {$c < [$Win(tbl) cget -columns]} {incr c} {
    # checkbox variable name is the same as the window name (wn)
    # the name is global
    set wn [winfo name [$Win(tbl) window cget $gridRow(Visible),$c -window]]
    global $wn
    if {[set $wn]} {
      set f [$Win(tbl) get $gridRow(Field),$c]
      if {$f == {}} {continue}
      set a [$Win(tbl) get $gridRow(Alias),$c]
      set t [$Win(tbl) get $gridRow(Table),$c]
      if {$sqlstart == 1} {
    set sqlcmd "$sqlcmd, "
      }
      if {$h != 0} {
    set fs [checkFunction $c]
    if {$fs != ""} {
      set fe ")"
    } else {
      set fe ""
    }
      } else {
    set fs ""
    set fe ""
      }

      set i [string index $f 0]

      if {$i == "="} {
    set f [string range $f 1 end]
    if {$a == ""} {
      append sqlcmd "$fs$f$fe"
    } else {
      append sqlcmd "$fs$f$fe AS \"$a\""
    }
      } else {
    if {$a == ""} {
      append sqlcmd "$fs\"${t}\".\"${f}\"$fe"
    } else {
      append sqlcmd "$fs\"${t}\".\"${f}\"$fe AS \"$a\""
    }
      }
      set sqlstart 1
    }
    }

    # -------------- FROM section -----------------------
    # Read table names from vqb(tid)
    set tables [list]
    foreach lst $vqb(tid) {
    set t [lindex $lst 0]
    set t [$t.lab cget -text]
    lappend tables "$t"
    }

    # Check there are at least n-1 links, where n = number of tables
    set n [llength $tables]
    if {[llength $vqb(links)] < [expr $n - 1]} {
    tk_messageBox -message "There should be at least n-1 links, where n = no. of tables" -type ok
    return $sqlcmd
    }

    switch -- $PgAcVar(properties,FROM) {
    0 {set sup1 [FROMusingWHERE $tables $n]}
    1 {set sup1 [FROMusingJOIN $tables]}
    }
    append sqlcmd $sup1
    # -------------- End FROM section -----------------------

    # -------------- WHERE or GROUP BY ----------------------
    set h [$Win(tbl) height $gridRow(Function)]
    if {$h != 0} {
    set sup [computeGROUP]
    } else {
    set sup [computeWHERE]
    }
    append sqlcmd $sup
    # -------------- End WHERE or GROUP BY ------------------

    # -------------- ORDER BY -------------------------------
    set sup2 {}
    for {set c 1} {$c < [$Win(tbl) cget -columns]} {incr c} {
    set how [$Win(tbl) get $gridRow(Sort),$c]
    if {$how != ""} {
      if {$how != "Unsorted"} {
    if {$how == "Ascending"} {
      set how ASC
    } else {
      set how DESC
    }
    if {$sup2 == ""} {
      set sup2 "\nORDER BY "
    } else {
      append sup2 ","
    }

    # If field starts with an equals sign (=) then assume that
    # text in the field cell is an expression, so add contents
    # of the field (minus '=' sign, and no quotess) to sqlcmd
    set f [$Win(tbl) get $gridRow(Field),$c]
    set i [string index $f 0]
    if {$i == "="} {
      set f [string range $f 1 end]
      append sup2 "${f} $how "
    } else {
      set t [$Win(tbl) get $gridRow(Table),$c]
      append sup2 "\"${t}\".\"${f}\" $how "
    }
      }
    }
    }
    append sqlcmd " $sup2"
    # -------------- End ORDER BY section -------------------

    # -------------- LIMIT, OFFSET --------------------------
    set sup3 {}
    if {$PgAcVar(properties,LIMIT) > 0} {
    set sup3 "\nLIMIT $PgAcVar(properties,LIMIT)"
    if {$PgAcVar(properties,OFFSET) > 0} {
      append sup3 " OFFSET $PgAcVar(properties,OFFSET)"
    }
    append sqlcmd " $sup3"
    }
    # -------------- End LIMIT, OFFSET section --------------

    return $sqlcmd
    #return [set vqb(qcmd) $sqlcmd]

}; # end proc ::VisualQB::computeSELECT


#----------------------------------------------------------
# ::VisualQB::computeGROUP
#----------------------------------------------------------
#
proc ::VisualQB::computeGROUP {} {

    global PgAcVar
    variable vqb
    variable gridRow
    variable Win

    set addSectionStart 0
    set grpStart 0
    set group {}

    set crit [computeCRITERIA 1]
    if {$crit != ""} {
    append group "\nWHERE ($crit"
    }

    for {set c 1} {$c < [$Win(tbl) cget -columns]} {incr c} {
    set s [string tolower [string trim [$Win(tbl) get $gridRow(Function),$c]]]
    switch $s {
      "group by" -
      "having" -
      "where" {
            if {$grpStart == 1} {
              append group ", "
            }
            if {$addSectionStart == 0} {
              append group "\nGROUP BY "
            }

            # If field starts with an equals sign (=) then assume that
            # text in the field cell is an expression, so add contents
            # of the field (minus '=' sign, and no quotess) to sqlcmd
            set f [$Win(tbl) get $gridRow(Field),$c]
            set i [string index $f 0]
            if {$i == "="} {
              set f [string range $f 1 end]
              set isExpression 1
            } else {
              set isExpression 0
            }
            set t [$Win(tbl) get $gridRow(Table),$c]
            if {$isExpression == 1} {
              append group "${f}"
            } else {
              append group "\"${t}\".\"${f}\""
            }
            set grpStart 1
            set addSectionStart 1
          }
    }
    }

    set crit [computeCRITERIA 2]
    if {$crit != ""} {
    append group "\nHAVING ($crit"
    }

    return $group
}; # end proc ::VisualQB::computeGROUP


#----------------------------------------------------------
# ::VisualQB::computeWHERE
#----------------------------------------------------------
#
proc ::VisualQB::computeWHERE {} {

    global PgAcVar
    variable vqb
    variable gridRow
    variable Win

    # -------------- WHERE section -----------------------
    # Expressions in multiple columns in a single row are treated as 'And' criteria. To be
    # selected as part of the query's results, a record must meet all the criteria in a given row.
    # Expressions in different rows are treated as 'Or' criteria. To be selected, a record needs
    # to meet the criteria only in any one row.
    #--------------------------------------------------------------------------
    set where {}

    # set start of this section to 'where' or 'and'
    if {$PgAcVar(properties,FROM) == 1 || [llength $vqb(links)] == 0} {
    set strt "\nWHERE ("
    } else {
    set strt " AND ("
    }
    set c [computeCRITERIA 3]
    if {$c != ""} {
    append where "$strt$c"
    }

    return $where
    # -------------- End WHERE section ----------------------

}; # end proc ::VisualQB::computeWHERE


#----------------------------------------------------------
# ::VisualQB::computeCRITERIA
# 
# arg
#  1 = only add to criteria when Function row contains 'where'
#  2 = only add to criteria when Function row contains 'having'
#      or a function
#  3 = add all to criteria (Function row is hidden)
#----------------------------------------------------------
#
proc ::VisualQB::computeCRITERIA {arg} {

    global PgAcVar
    variable vqb
    variable gridRow
    variable Win

    set vqb(rescriteria) {}

    set h [$Win(tbl) height $gridRow(Function)]
    set crit {}
    set addSectionStart 0
    # 1 = By row, 0 = By col
    switch -- $PgAcVar(properties,CRITERIA) {
    1 {
    for {set r $gridRow(Criteria)} {$r < [$Win(tbl) cget -rows]} {incr r} {
      set str "("
      for {set c 1} {$c < [$Win(tbl) cget -columns]} {incr c} {
        if {[string trim [$Win(tbl) get $r,$c]] != ""} {

          # If field starts with an equals sign (=) then assume that
          # text in the field cell is an expression, so add contents
          # of the field (minus '=' sign, and no quotess) to sqlcmd
          set f [$Win(tbl) get $gridRow(Field),$c]
          set i [string index $f 0]
          if {$i == "="} {
            set f [string range $f 1 end]
            set isExpression 1
          } else {
            set isExpression 0
          }
          set t [$Win(tbl) get $gridRow(Table),$c]
          set o [$Win(tbl) get $r,$c]
          #set o [queryParams $o]
          if {$h != 0} {
            set fs [checkFunction $c]
            if {$fs != ""} {
              set fe ")"
            } else {
              set fe ""
            }
          } else {
            set fs ""
            set fe ""
          }
          set rt [string tolower [string trim [$Win(tbl) get $gridRow(Function),$c]]]
          switch $arg {
            1 {
                if {$rt == "where"} {
          set o [queryParams $o]
                  if {$isExpression == 1} {
                    append str "($fs${f}$fe $o) AND "
                  } else {
                    append str "($fs\"${t}\".\"${f}\"$fe $o) AND "
                  }
                }
              }
            2 {
                if {$rt != "where" && $rt != "group by"} {
          set o [queryParams $o]
                  if {$isExpression == 1} {
                    append str "($fs${f}$fe $o) AND "
                  } else {
                    append str "($fs\"${t}\".\"${f}\"$fe $o) AND "
                  }
                }
              }
            3 {
          set o [queryParams $o]
                if {$isExpression == 1} {
                  append str "($fs${f}$fe $o) AND "
                } else {
                  append str "($fs\"${t}\".\"${f}\"$fe $o) AND "
                }
              }
          }
        }
      }

      # remove last " and " if there is one
      set pos [string last " AND " $str]
      if {$pos > 0} {
        set str [string range $str 0 [expr $pos - 1]]
      }
      append str ")"
      if {$str != "()"} {
        set addSectionStart 1
      }
      lappend vqb(rescriteria) $str
    }
      }
    0 {
    for {set c 1} {$c < [$Win(tbl) cget -columns]} {incr c} {
      set str "("
      for {set r $gridRow(Criteria)} {$r < [$Win(tbl) cget -rows]} {incr r} {
        if {[string trim [$Win(tbl) get $r,$c]] != ""} {

          # If field starts with an equals sign (=) then assume that
          # text in the field cell is an expression, so add contents
          # of the field (minus '=' sign, and no quotess) to sqlcmd
          set f [$Win(tbl) get $gridRow(Field),$c]
          set i [string index $f 0]
          if {$i == "="} {
            set f [string range $f 1 end]
            set isExpression 1
          } else {
            set isExpression 0
          }
          set t [$Win(tbl) get $gridRow(Table),$c]
          set o [$Win(tbl) get $r,$c]
          #set o [queryParams $o]
          if {$h != 0} {
            set fs [checkFunction $c]
            if {$fs != ""} {
              set fe ")"
            } else {
              set fe ""
            }
          } else {
            set fs ""
            set fe ""
          }
          set rt [string tolower [string trim [$Win(tbl) get $gridRow(Function),$c]]]
          switch $arg {
            1 {
                if {$rt == "where"} {
          set o [queryParams $o]
                  if {$isExpression == 1} {
                    append str "($fs${f}$fe $o) OR "
                  } else {
                    append str "($fs\"${t}\".\"${f}\"$fe $o) OR "
                  }
                }
              }
            2 {
                if {$rt != "where" && $rt != "group by"} {
          set o [queryParams $o]
                  if {$isExpression == 1} {
                    append str "($fs${f}$fe $o) OR "
                  } else {
                    append str "($fs\"${t}\".\"${f}\"$fe $o) OR "
                  }
                }
              }
            3 {
          set o [queryParams $o]
                if {$isExpression == 1} {
                  append str "($fs${f}$fe $o) OR "
                } else {
                  append str "($fs\"${t}\".\"${f}\"$fe $o) OR "
                }
            }
          }
        }
      }
      # remove last " OR " if there is one
      set pos [string last " OR " $str]
      if {$pos > 0} {
        set str [string range $str 0 [expr $pos - 1]]
      }
      append str ")"
      if {$str != "()"} {
        set addSectionStart 1
      }
      lappend vqb(rescriteria) $str
    }
    }
    }


    if {$addSectionStart == 1} {
    set addor 0
    #append crit $strt

    foreach s $vqb(rescriteria) {
      if {$s != "()"} {
    if {$addor == 1} {
      if {$PgAcVar(properties,CRITERIA) == 1} {
        append crit "\nOR "
      } else {
        append crit "\nAND "
      }
    }
    append crit $s
    set addor 1
      }
    }
    append crit ")"
    }
    return $crit
}; # end proc ::VisualQB::computeCRITERIA


#----------------------------------------------------------
# ::VisualQB::FROMusingJOIN
#----------------------------------------------------------
#
proc ::VisualQB::FROMusingJOIN {tables} {

    global PgAcVar
    variable vqb

    set Win(tbl)s {}
    set done {}
    set from1 {}
    set first 1

    # When only one table still need a FROM
    if {[llength $vqb(links)] == 0} {
    set sn [lindex $tables 0]
    append from1 "\nFROM \"$sn\" "
    } else {
    # Add all tables in links to list
    foreach lnk $vqb(links) {
      set source [lindex $lnk 0]
      set target [lindex $lnk 2]
      lappend Win(tbl)s $source $target  
    }
    # Sort the list using unique
    lsort -unique $Win(tbl)s
    # For each name in Win(tbl)s, search for name
    foreach t $Win(tbl)s {
      foreach lnk $vqb(links) {
    foreach {st sf tt tf jt} $lnk {
      if {$t == $st || $t == $tt} {
        # Get the database table name (from the canvas table name)
        set sn [$st.lab cget -text]
        set tn [$tt.lab cget -text]
        # Add link to FROM clause
        if {$first == 1} {
          set first 0
          append from1 "\nFROM \"$sn\" $jt JOIN \"$tn\" ON (\"$sn\".\"$sf\" = \"$tn\".\"$tf\")"
          lappend done $st $tt
        } else {
          set sd [lsearch $done $st]           ;# sd - Is source table in done? i.e. >=0
          set td [lsearch $done $tt]           ;# td - Is target table in done? i.e. >=0
          if {$td >= 0 && $sd >= 0} {
            continue
          } elseif {$td == -1 && $sd == -1} {
            continue                           ;# this is actually an error and should not happen
          } elseif {$td == -1 && $sd >= 0} {
            append from1 " $jt JOIN \"$tn\" ON (\"$sn\".\"$sf\" = \"$tn\".\"$tf\")"
            lappend done $tt
          } elseif {$sd == -1 && $td >= 0} {
            append from1 " $jt JOIN \"$sn\" ON (\"$sn\".\"$sf\" = \"$tn\".\"$tf\")"
            lappend done $st
          }
        }
      }
    }
      }
    }
    }
    return $from1

}; # end proc ::VisualQB::FROMusingJOIN


#----------------------------------------------------------
# ::VisualQB::FROMusingWHERE
#----------------------------------------------------------
#
proc ::VisualQB::FROMusingWHERE {tables n} {

    global PgAcVar
    variable vqb

    set from1 {}
    if {$n > 0} {
    set from1 "\nFROM [join $tables ,] "
    }

    set from2 {}
    if {[llength $vqb(links)] > 0} {
    set from2 "\nWHERE "
    foreach link $vqb(links) {
      if {$from2 != "\nWHERE "} {
    append from2 " AND "
      }
      foreach {t1 f1 t2 f2 jt} $link {
    set tab1 [$t1.lab cget -text]
    set tab2 [$t2.lab cget -text]
    append from2 " (\"${tab1}\".\"${f1}\"=\"${tab2}\".\"${f2}\")"
      }
    }
    }
    return [append from1 $from2]

}; # end proc ::VisualQB::FROMusingWHERE


#----------------------------------------------------------
# ::VisualQB::checkFunction
#
#  Check to see if there is a function in the Function row
#  If yes return the function name and opening parenthesis
#----------------------------------------------------------
#
proc ::VisualQB::checkFunction {c} {

    global PgAcVar
    variable vqb
    variable gridRow
    variable Win

    set rt [string tolower [string trim [$Win(tbl) get $gridRow(Function),$c]]]
    switch $rt {
    "avg" -
    "avg()"      {set fs "avg("}
    "count" -
    "count()"    {set fs "count("}
    "count(*)"   {set fs "count(*"}
    "max" -
    "max()"      {set fs "max("}
    "min" -
    "min()"      {set fs "min("}
    "stddev" -
    "stddev()"   {set fs "stddev("}
    "sum" -
    "sum()"      {set fs "sum("}
    "variance" -
    "variance()" {set fs "variance("}
    default      {set fs ""}
    }
    return $fs
}; # end proc ::VisualQB::checkFunction


#----------------------------------------------------------
# ::VisualQB::queryParams (s)
#
#   Check string s for user input in a query. User input is
#   surrounded with brackets ([]).
#   If there are no brackets in the string then return the
#   string unaltered.
#   If there are brackets then prompt the user for input
#   with the text between the brackets as the prompt. Then
#   replace the brackets and text within the brackets with
#   the user input and return the altered string.
#   e.g. if you enter the following string in criteria
#       like '[Name:]'
#   you will be prompted with 'Name:'. If you enter S% the
#   following string will be returned
#       like 'S%'
#----------------------------------------------------------
#
proc ::VisualQB::queryParams {s} {
    set str $s

    # Octal number 133 is left bracket ([)
    set start [string first "\133" $str]
    if {$start == -1} {
    return $str
    }
    
    # Octal number 135 is right bracket (])
    set finish [string first "\135" $str]
    if {$finish == -1} {
    tk_messageBox -message "There is an opening bracket but no matching closing bracket" -type ok
    return $str
    }

    set prompt [string range $str [expr $start + 1] [expr $finish - 1]]
    set res [Dialog_Prompt "Query Parameter" $prompt]

    if {$res != {}} {
    set str [string replace $str $start $finish $res]
    # Recurse the string in case of further brackets
    set str [queryParams $str]
    } else {
    return $str
    }

}; # end proc ::VisualQB::queryParama

#-------------------------------------------------------------
# Display a prompt window with a text entry widget
# if OK button clicked, return text entry
# else if Cancel button clicked, return empty string
#-------------------------------------------------------------
#
proc ::VisualQB::Dialog_Prompt {title str} {
    global prompt

    set prompt(ok) 0
    set f .prompt
    toplevel $f -borderwidth 10
    wm title $f [intlmsg $title]
    message $f.msg -text $str -aspect 1000
    entry $f.entry -textvariable prompt(result)
    set b [frame $f.buttons]
    pack $f.msg $f.entry $f.buttons -side top -fill x
    pack $f.entry -pady 5
    button $b.ok -text OK -command {set prompt(ok) 1 ; destroy .prompt}
    button $b.cancel -text Cancel -command {set prompt(ok) 0 ; destroy .prompt}
    pack $b.ok -side left
    pack $b.cancel -side right
    bind $f.entry <Return> {set prompt(ok) 1}
    bind $f.entry <Control-c> {set prompt(ok) 0}

    # Set focus to toplevel window
    focus $f.entry
    # Then redirect all input to toplevel window
    grab $f
    # Then wait until toplevel window is destroyed before proceeding
    tkwait window $f
    
    if {$prompt(ok)} {
    return $prompt(result)
    } else {
    return {}
    }
}


#----------------------------------------------------------
# ::VisualQB::deleteObject --
#
#   handles deletion of any item in any part of the window
#----------------------------------------------------------
#
proc ::VisualQB::deleteObject {} {

    global PgAcVar
    variable vqb
    variable gridRow
    variable Win

    switch $vqb(hlite) {
        grid {
            $Win(tbl) delete [$Win(tbl) curselection]
        }
        link {
            set lnks [$Win(cv) find withtag hili]
            if {$lnks==""} return
            set taglist [$Win(cv) gettags [lindex $lnks 0]]
            if {$taglist==""} return
            set lt [lindex $taglist 1]
            set tagpos [string range $lt 4 end]
            set vqb(links) [lreplace $vqb(links) $tagpos $tagpos]
            $Win(cv) delete hili
            drawLinks
        }
        tabl {
            set it [$vqb(lbnam) itemcget [$vqb(lbnam) selection get] -data]
            set tablename [string trim $vqb(tablename$it) \"]

            # First, delete row(s) in query grid
            set i 0
            set rows [list]
            foreach col [$Win(tbl) getcolumns colTable] {
                if { $col == $tablename } {
                    lappend rows $i
                }
                incr i
            }
            $Win(tbl) delete $rows

            # Second, find and delete links to the table
            set tnam [winfo parent $vqb(lbnam)]
            set tagpos 0
            # Reverse the loop
            # Does the following work?
            foreach lnk $vqb(links) {
              if {[lindex $lnk 0]==$tnam || [lindex $lnk 2]==$tnam} {
                set vqb(links) [lreplace $vqb(links) $tagpos $tagpos]
              } else {
                incr tagpos
              }
            }
            $Win(cv) delete links
            drawLinks

            # Finally, delete the table
            set tid [VisualQB::gettid $tnam]
            $Win(cv) delete $tid
            # delete from vqb(tid)
            for {set pos 0} {$pos < [llength $vqb(tid)]} {incr pos} {
                set lst [lindex $vqb(tid) $pos]
                if { [lindex $lst 0] == $tnam } {
                    set vqb(tid) [lreplace $vqb(tid) $pos $pos]
                    break
                }
            }
        }
    }; # end switch

}; # end proc ::VisualQB::deleteObject


#----------------------------------------------------------
# ::VisualQB::deselectGrid --
#
#   removes all current cell selection
#----------------------------------------------------------
#
proc ::VisualQB::deselectGrid {} {
    variable Win
    $Win(tbl) cellselection clear 0,0 end,end
}; # end proc ::VisualQB::deselectGrid


#----------------------------------------------------------
# ::VisualQB::deselectLinks
#----------------------------------------------------------
#
proc ::VisualQB::deselectLinks {} {
    variable Win
    variable vqb

    set lnks [$Win(cv) find withtag hili]
    if {$lnks==""} {return}

    foreach i $lnks {
        $Win(cv) itemconfigure $i -fill black
        $Win(cv) dtag $i hili
    }

}; # end proc ::VisualQB::deselectLinks


#----------------------------------------------------------
# ::VisualQB::deselectTables
#----------------------------------------------------------
#
proc ::VisualQB::deselectTables {} {

    variable vqb

    foreach l $vqb(tid) {
    set w [lindex $l 0]
    $w.lb selection clear 0 end
    }  
}; # end proc ::VisualQB::deselectTables


#----------------------------------------------------------
# ::VisualQB::getTableList
#----------------------------------------------------------
#
proc ::VisualQB::getTableList {} {

    global PgAcVar
    variable vqb

    set tablelist {}
    set num 0
    foreach lst $vqb(tid) {
      set f [lindex $lst 0]
      set name [$f.lab cget -text]
      lappend tablelist $name [winfo x $f] [winfo y $f] t$num
      incr num
    }

    return $tablelist

}; # end proc ::VisualQB::getTableList


#----------------------------------------------------------
# ::VisualQB::getLinkList
#----------------------------------------------------------
#
proc ::VisualQB::getLinkList {} {

    global PgAcVar
    variable vqb

    set linklist {}
    foreach lst $vqb(links) {
    lappend linklist [lindex $lst 0] [lindex $lst 1] [lindex $lst 2] [lindex $lst 3] [lindex $lst 4]
    }

    return $linklist

}; # end proc ::VisualQB::getLinkList {} {


#----------------------------------------------------------
# ::VisualQB::loadVisualLayout
#----------------------------------------------------------
#
proc ::VisualQB::loadVisualLayout {} {

    global PgAcVar tbquery
    variable vqb
    variable gridRow
    variable Win

    # If first element of list is not a number then assume
    # this is a previous version query.
    set lst $PgAcVar(query,results)
    set isNew [string is digit [lindex $lst 0]]

    init
    foreach {t x y a} $PgAcVar(query,tables) {
        set vqb(newtablename) $t
        if {$t != {}} {
          addNewTable $x $y $a
        }
    }

    if {$isNew} {
      foreach {t0 f0 t1 f1 jt} $PgAcVar(query,links) {
    lappend vqb(links) [list $t0 $f0 $t1 $f1 $jt]
      }
    } else {
      foreach {t0 f0 t1 f1} $PgAcVar(query,links) {
    set fnam0 [string range $t0 1 end]
    set fnam1 [string range $t1 1 end]
    set tablename0 $Win(cv).tab$fnam0
    set tablename1 $Win(cv).tab$fnam1
    lappend vqb(links) [list $tablename0 $f0 $tablename1 $f1 "INNER"]
      }
    }

    # update required so drawlinks works correctly
    update
    drawLinks

    if {$PgAcVar(query,results) != ""} {
      set lst $PgAcVar(query,results)
      if {$isNew} {
    $Win(tbl) configure -rows [lindex $lst 0]
    $Win(tbl) configure -columns [lindex $lst 1]
    array set tbquery [lindex $lst 2]
    array set PgAcVar [lindex $lst 3]
    set h [lindex $lst 4]

    # Set height of Function row
    $Win(tbl) height $gridRow(Function) $h

    # Set checkbuttons to on or off
    for {set c 1} {$c < [$Win(tbl) cget -columns]} {incr c} {
      if {[$Win(tbl) get $gridRow(Visible),$c] == "Yes"} {
        set wn [$Win(tbl) window cget $gridRow(Visible),$c -window]
        $wn select
      }
    }
    drawLinks
      } else {
    set cl 1
    foreach {f t s c r} $PgAcVar(query,results) {
      $Win(tbl) set 1,$cl $f
      set tabnum [string range $t 1 end]
      # Remove leading and trailing double-quotes
      set tablename [string trim $vqb(tablename$tabnum) \"]
      $Win(tbl) set 2,$cl $tablename
      if {$s == "unsorted"} {
        set s Unsorted
      }
      $Win(tbl) set 3,$cl $s
      $Win(tbl) set 4,$cl $r          
      $Win(tbl) set 5,$cl $c
      incr cl
      if {$cl > [expr [$Win(tbl) cget -columns] - 1]} {
        $Win(tbl) insert cols [expr $cl - 1] 1
      }
    }
      }
    }

}; # end proc ::VisualQB::loadVisualLayout


#----------------------------------------------------------
# ::VisualQB::repaintAll
#----------------------------------------------------------
#
proc ::VisualQB::repaintAll {} {

    global PgAcVar
    variable vqb
    variable Win

    $Win(cv) delete all
    set posx 20

    foreach tn [array names vqb tablename*] {
        regsub tablename $tn "" it
        drawTable $it
    }

    drawLinks

    bind .pgaw:VisualQuery <Key-Delete> {
    VisualQB::deleteObject
    }

}; # end proc ::VisualQB::repaintAll


#----------------------------------------------------------
# ::VisualQB::showSQL
#----------------------------------------------------------
#
proc ::VisualQB::showSQL {} {

    global PgAcVar
    variable vqb

    set sqlcmd [computeSQL]
    set tl .showSQL
    toplevel $tl -class Toplevel
    wm title $tl [intlmsg "Show SQL"]
    text $tl.txtSQL -height 15 -width 80 -bg white -wrap word
    $tl.txtSQL insert end $sqlcmd
    $tl.txtSQL configure -state disabled
    button $tl.close -text "Close" -command {destroy .showSQL}
    pack $tl.txtSQL -in $tl -fill both
    pack $tl.close -in $tl -fill x

}; # end proc ::VisualQB::showSQL


#----------------------------------------------------------
# ::VisualQB::saveToQueryBuilder
#----------------------------------------------------------
#
proc ::VisualQB::saveToQueryBuilder {} {

    global PgAcVar tbquery
    variable vqb
    variable gridRow
    variable Win

    Window show .pgaw:QueryBuilder
    $::Queries::Win(qrytxt) delete 1.0 end
    set vqb(qcmd) [computeSQL]
    set PgAcVar(query,tables) [getTableList]
    set PgAcVar(query,links) [getLinkList]

    # Get height of Function row
    set h [$Win(tbl) height $gridRow(Function)]
    
    # Synch visible row with checkbuttons
    for {set c 1} {$c < [$Win(tbl) cget -columns]} {incr c} {
      set wn [winfo name [$Win(tbl) window cget $gridRow(Visible),$c -window]]
      global $wn
      if {[set $wn]} {
    $Win(tbl) set $gridRow(Visible),$c "Yes"
      } else {
    $Win(tbl) set $gridRow(Visible),$c "No"
      }
    }
    
    # 'results' stores no. of rows and columns in grid, contents of the grid, properties of the query and height of Function row
    set PgAcVar(query,results) [list [$Win(tbl) cget -rows] [$Win(tbl) cget -columns] [array get tbquery] [array get PgAcVar "properties,*"] $h]
    $::Queries::Win(qrytxt) insert end $vqb(qcmd)
    focus .pgaw:QueryBuilder

    return

}; # end proc ::VisualQB::saveToQueryBuilder


#----------------------------------------------------------
# ::VisualQB::executeSQL
#----------------------------------------------------------
#
proc ::VisualQB::executeSQL {} {

    global PgAcVar
    variable vqb

    set vqb(qcmd) [computeSQL]
    set wn [Tables::getNewWindowName]
    set PgAcVar(mw,$wn,query) [subst $vqb(qcmd)]
    set PgAcVar(mw,$wn,updatable) 0
    set PgAcVar(mw,$wn,isaquery) 1
    set Tables::Win($wn,updatable) 0
    set Tables::Win($wn,isaquery) 1
    set Tables::Win($wn,ukey) ""
    Tables::createWindow $wn 1
    Tables::loadLayout $wn $PgAcVar(mw,$wn,query)
    Tables::displayWindow $wn
    Tables::selectRecords $wn $PgAcVar(mw,$wn,query)

}; # end proc ::VisualQB::executeSQL


#----------------------------------------------------------
# ::VisualQB::editProperties
#----------------------------------------------------------
#
proc ::VisualQB::editProperties {} {

    global PgAcVar
    variable vqb

    set tl .eProp
    toplevel $tl -class Toplevel
    wm title $tl [intlmsg "Query Properties"]

    set w $tl.f
    frame $w
    pack $w -side top -expand 1 -fill both

    label $w.ld -text "DISTINCT" -width 11 -anchor w
    set r $tl.f.rf
    frame $r
    radiobutton $r.ry \
    -variable PgAcVar(properties,DISTINCT) -value 1 -text "Yes" -relief sunken -width 5
    radiobutton $r.rn \
    -variable PgAcVar(properties,DISTINCT) -value 0 -text "No" -relief sunken -width 5
    pack $r.ry $r.rn -side left -expand 1 -fill both

    label $w.lon -text "ON" -width 11 -anchor e
    entry $w.eon -textvariable PgAcVar(properties,ON) -relief sunken -width 12

    label $w.fj -text "FROM -  Use" -width 11 -anchor w
    set j $tl.f.jf
    frame $j
    radiobutton $j.rj \
    -variable PgAcVar(properties,FROM) -value 1 -text "Join" -relief sunken -width 5
    radiobutton $j.rw \
    -variable PgAcVar(properties,FROM) -value 0 -text "Where" -relief sunken -width 5
    pack $j.rj $j.rw -side left -expand 1 -fill both

    label $w.fc -text "CRITERIA" -width 11 -anchor w
    set c $tl.f.cf
    frame $c
    radiobutton $c.rr \
    -variable PgAcVar(properties,CRITERIA) -value 1 -text "Row" -relief sunken -width 5
    radiobutton $c.rc \
    -variable PgAcVar(properties,CRITERIA) -value 0 -text "Col" -relief sunken -width 5
    pack $c.rr $c.rc -side left -expand 1 -fill both

    label $w.ll -text "LIMIT" -width 11 -anchor w
    entry $w.el -textvariable PgAcVar(properties,LIMIT) -relief sunken -width 12

    label $w.loff -text "OFFSET"  -width 11 -anchor e
    entry $w.eoff -textvariable PgAcVar(properties,OFFSET) -relief sunken -width 12

    #pack $w.ld $r -side top -expand 1 -fill both
    #pack $w.lon $w.eon -side top -expand 1 -fill both
    grid $w.ld $r -padx 0 -pady 1 -sticky news
    grid $w.lon $w.eon -padx 1 -pady 1 -sticky news
    grid $w.fj $j -padx 0 -pady 1 -sticky news
    grid $w.fc $c -padx 0 -pady 1 -sticky news
    grid $w.ll $w.el -padx 0 -pady 1 -sticky news
    grid $w.loff $w.eoff -padx 0 -pady 1 -sticky news

    button $tl.close -text "Close" -command {destroy .eProp}
    pack $tl.close -in $tl -fill x

}; # end proc ::VisualQB::editProperties


#----------------------------------------------------------
# ::VisualQB::editJoinType
#----------------------------------------------------------
#
proc ::VisualQB::editJoinType {} {

    global PgAcVar
    variable vqb

    set tl .eProp
    toplevel $tl -class Toplevel
    wm title $tl [intlmsg "Join Type"]

    set w $tl.f
    frame $w
    pack $w -side top -expand 1 -fill both

    # Find the join type of the currently selected link
    set idx $vqb(CurrentLink)
    set lnk [lindex $vqb(links) $idx]
    set typ [lindex $lnk end]
    switch $typ {
    INNER {set PgAcVar(properties,JoinType) 0}
    LEFT {set PgAcVar(properties,JoinType) 1}
    RIGHT {set PgAcVar(properties,JoinType) 2}
    FULL {set PgAcVar(properties,JoinType) 3}
    }

    radiobutton $w.ri \
    -variable PgAcVar(properties,JoinType) \
    -value 0 \
    -justify left \
    -relief sunken \
    -anchor w \
    -command {VisualQB::changeJoinType INNER} \
    -text [intlmsg "INNER join: An inner join will discard any row for which it
cannot find a corresponding value between the sets being joined."]
    radiobutton $w.rl \
    -variable PgAcVar(properties,JoinType) \
    -value 1 \
    -justify left \
    -relief sunken \
    -anchor w \
    -command {VisualQB::changeJoinType LEFT} \
    -text [intlmsg "LEFT outer join: A left outer join will always return at least
one instance of each row in the set of rows to the left of the
JOIN keyword. Missing columns in the right set are populated
with NULL values."]
    radiobutton $w.rr \
    -variable PgAcVar(properties,JoinType) -value 2 -justify left -relief sunken -anchor w\
    -command {VisualQB::changeJoinType RIGHT} \
-text "RIGHT outer join: A right outer join will always return at least
one instance of each row in the set of rows to the right of the
JOIN keyword. Missing columns in the left set are populated
with NULL values."
    radiobutton $w.rf \
    -variable PgAcVar(properties,JoinType) -value 3 -justify left -relief sunken -anchor w\
    -command {VisualQB::changeJoinType FULL} \
-text "FULL outer join: A full outer join will always return at least
one instance of each row in each joined set. Missing columns on
either side will be populated with NULL values."
    pack $w.ri -side top -expand 1 -fill both
    pack $w.rl -side top -expand 1 -fill both
    pack $w.rr -side top -expand 1 -fill both
    pack $w.rf -side top -expand 1 -fill both

    button $tl.close -text "Close" -command {destroy .eProp}
    pack $tl.close -in $tl -fill x

}; # end proc ::VisualQB::editJoinType


#----------------------------------------------------------
# ::VisualQB::changeJoinType
#----------------------------------------------------------
#
proc ::VisualQB::changeJoinType {typ} {

    global PgAcVar
    variable vqb

    set idx $vqb(CurrentLink)
    set lnk [lindex $vqb(links) $idx]

    set lnk [lreplace $lnk end end $typ]
    set vqb(links) [lreplace $vqb(links) $idx $idx $lnk]

}; # end proc ::VisualQB::changeJoinType


#----------------------------------------------------------
# ::VisualQB::tableButton1
#----------------------------------------------------------
#
proc ::VisualQB::tableButton1 {w x y} {
    variable vqb

    # deselect everything else
    deselectLinks
    deselectTables

    # make sure we know we are on the grid
    set vqb(hlite) grid

}; # end proc ::VisualQB::tableButton1


#----------------------------------------------------------
# ::VisualQB::tableButton3
#
#   Show popup menu on grid, if something is selected
#----------------------------------------------------------
#
proc ::VisualQB::tableButton3 {w x y} {
    variable Win

    if { [$Win(tbl) curcellselection] != "" } {
        tk_popup $Win(tbl).pop [winfo pointerx $w] [winfo pointery $w]
    }

}; # end proc ::VisualQB::tableButton3 


#==========================================================
#   GUI
#==========================================================
#
proc vTclWindow.pgaw:VisualQuery {base} {

    global PgAcVar cbw
    variable ::VisualQB::vqb
    variable ::VisualQB::Win

    if {$base==""} {
    set base .pgaw:VisualQuery
    }

    if {[winfo exists $base]} {
    wm deiconify $base; return
    }

    toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 759x530+10+13
    wm maxsize $base 1280 1024
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base [intlmsg "Visual Query Builder"]

    bind $base <Key-F1> "Help::load visual_designer"
    bind $base <Key-Delete> {VisualQB::deleteObject}

    # frame widget for label and combobox and buttonbox
    frame $base.fb \
        -height 75 \
        -width 125

    # label and combobox widgets to add a table
    Label $base.fb.ltable \
        -padx 5 \
        -text [intlmsg "Add table"]
    ComboBox $base.fb.cbtable \
        -values [concat [::Database::getPrefObjList Tables] \
            [::Database::getPrefObjList Views]] \
        -editable true \
        -textvariable ::VisualQB::vqb(newtablename) \
        -modifycmd {::VisualQB::addNewTable}

    set ::VisualQB::Win(entertable) $base.fb.cbtable

    image create photo properties-16 -data {
      R0lGODlhEAAQAPcAAAQCBMTCxAQChPz+/GsEAhUAAPUAAHcAAAADAFkAABUA
      AAAAADQAAAAAAAAAAMAAAAAAAAAAAAAAAAAAAGUEACIAAAAAAAAAAB0DASIA
      AAAAAAAAAHMAGgAAAAAAAAAAAFYAAAAAAAAAAAAAALAEAAIAAAAAAAAAAAAD
      FQEAAAAAAAAAAPwEFeUAABIAAAAAAFMCDSEAAOcAAHcAAAAADQAAAAEAAAAA
      AAAAPAAADhUAEAAASgQAa+UAAhIAhQAAAwAAFwEAAAAAAAAAACQBAeYAABIA
      AAAAAAUaGZAAAPcAAHcAAMgAEdUAAPYAAHcAAP8AZf8AHP8A1/8AWmoVhBYA
      APUAAHcAAMQVsB4A0OcA2XcAWgANFQAAABUAAAAAAAANDAAAAAAAAAAAACg8
      cY0OGxsQ1wBKWgRrmAECPwWFMwEDADgXdOMABxIAVwAAAH4BiAAAPwAAM8AA
      AAAZ9gAAGgAA1wAAWv8RhP8AAP8AAP8AAP8BbP8AW/8A5/8AdwAH+AAAwgAA
      FQAAAAAI+AAAwgAAFQAAAAAJ+AAAwgAAFQAAAGw8AABBwADUFQB3AGBkAAPA
      AADZAAB3AAbKB/ZcAObUAHd3AA+85B5cBOfUAHd3AACjAAGmAADYAABaAABA
      AFnnAAESAAAAAADIAAAoABXnAAB3AHAAAOUAABIAAAAAAAAAbAAAWwAA5wAA
      dzwAHvlZABIVAAAAAAUA/5AB//cA/3cA/8gEHtUpAPbnAHd3AP9gsP/n8v8S
      Ev8AAGpkABYvAPXoAHd3AOAAaBgA6OcAEncAAAD4/gDCWBUVSwAAAABvSAAQ
      6ADnEgB3AAD/W1n/YhX/TgD/AABkfAHo6AASEgAAAGSYby8QEOjn53d3dwBs
      AgBv6QBOEgAAACMo4ACW/wBO/wAAf/AA5OcB6BIAEgAAAIhg+D/nwjMSFQAA
      AAAAbwABEAAA5wAAdwDA+ADcwgFPFQAAAAEAAQAwAAIAAAAAAKAEAAAAAAAA
      AAAAAB8DbgAA/QAARgAAACH5BAAAAAAALAAAAAAQABAABwhTAAEIHEiwoMGD
      AgUIOBigoUMACg0GAODQYUSCEzNWjDhgAMWPGidy/OgxI0GOHjuWBOBx4EKV
      MFm2JAiz5syBLW+yNGgzZsGcP3n27IiwqFGjAQEAOw==
    }
    image create photo func-16 -data {
      R0lGODlhEAAQAPcAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDAwP8AAAD/
      AP//AAAA//8A/wD//////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
      AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
      AAAAMwAAZgAAmQAAzAAA/wAzAAAzMwAzZgAzmQAzzAAz/wBmAABmMwBmZgBm
      mQBmzABm/wCZAACZMwCZZgCZmQCZzACZ/wDMAADMMwDMZgDMmQDMzADM/wD/
      AAD/MwD/ZgD/mQD/zAD//zMAADMAMzMAZjMAmTMAzDMA/zMzADMzMzMzZjMz
      mTMzzDMz/zNmADNmMzNmZjNmmTNmzDNm/zOZADOZMzOZZjOZmTOZzDOZ/zPM
      ADPMMzPMZjPMmTPMzDPM/zP/ADP/MzP/ZjP/mTP/zDP//2YAAGYAM2YAZmYA
      mWYAzGYA/2YzAGYzM2YzZmYzmWYzzGYz/2ZmAGZmM2ZmZmZmmWZmzGZm/2aZ
      AGaZM2aZZmaZmWaZzGaZ/2bMAGbMM2bMZmbMmWbMzGbM/2b/AGb/M2b/Zmb/
      mWb/zGb//5kAAJkAM5kAZpkAmZkAzJkA/5kzAJkzM5kzZpkzmZkzzJkz/5lm
      AJlmM5lmZplmmZlmzJlm/5mZAJmZM5mZZpmZmZmZzJmZ/5nMAJnMM5nMZpnM
      mZnMzJnM/5n/AJn/M5n/Zpn/mZn/zJn//8wAAMwAM8wAZswAmcwAzMwA/8wz
      AMwzM8wzZswzmcwzzMwz/8xmAMxmM8xmZsxmmcxmzMxm/8yZAMyZM8yZZsyZ
      mcyZzMyZ/8zMAMzMM8zMZszMmczMzMzM/8z/AMz/M8z/Zsz/mcz/zMz///8A
      AP8AM/8AZv8Amf8AzP8A//8zAP8zM/8zZv8zmf8zzP8z//9mAP9mM/9mZv9m
      mf9mzP9m//+ZAP+ZM/+ZZv+Zmf+ZzP+Z///MAP/MM//MZv/Mmf/MzP/M////
      AP//M///Zv//mf//zP///yH5BAEAABAALAAAAAAQABAAAAiyAKdRcwaLmkCD
      f6YlHDXNGUGBDmGhciaIojNQ1FBBnDYNligUIEPCcvbHGbWCsEChEKQDBZ9/
      1FCYggWrIbWY/1CBNPXv38iGHREWBPXnJCo/A2/+QcHQ2UxU1IrWNAgLxTQU
      Awua7Li1IYqfNQmOpOrsI0idAwVenWhw4FcUzgi9JRRT6024WKeNYmoyb82q
      dZ2hWFoQ7ljBZVEQYljID9OpggvFlHqT4E2wNiE743owIAA7
    }

    # button widgets for toolbar
    ButtonBox $base.fb.bbox \
        -orient horizontal \
        -homogeneous 1 \
        -spacing 2
    $base.fb.bbox add \
        -relief link \
        -borderwidth 1 \
        -image func-16 \
        -name sum \
        -helptext [intlmsg "Show Functions"] \
        -command VisualQB::showFunctions
    $base.fb.bbox add \
        -relief link \
        -borderwidth 1 \
        -image properties-16 \
        -helptext [intlmsg "Show Properties"] \
        -command VisualQB::editProperties
    $base.fb.bbox add \
        -relief link \
        -borderwidth 1 \
        -image ::icon::imagegallery-22 \
        -helptext [intlmsg "Show SQL"] \
        -command VisualQB::showSQL
    $base.fb.bbox add \
        -relief link \
        -borderwidth 1 \
        -image ::icon::misc-16 \
        -helptext [intlmsg "Execute SQL"] \
        -command VisualQB::executeSQL
    $base.fb.bbox add \
        -relief link \
        -borderwidth 1 \
        -image ::icon::filesave-22 \
        -helptext [intlmsg "Save to query builder"] \
        -command VisualQB::saveToQueryBuilder
    $base.fb.bbox add \
        -relief link \
        -borderwidth 1 \
        -image ::icon::fileprint-22 \
        -helptext [intlmsg "Print"] \
        -command VisualQB::print
    $base.fb.bbox add \
        -relief link \
        -borderwidth 1 \
        -image ::icon::help-22 \
        -helptext [intlmsg "Help"] \
        -command {::Help::load visual_designer}
    $base.fb.bbox add \
        -relief link \
        -borderwidth 1 \
        -image ::icon::exit-22 \
        -helptext [intlmsg "Close"] \
        -command {Window destroy .pgaw:VisualQuery}

    # create paned window to hold canvas and table
    set pw1 [PanedWindow $base.pw -side left]
    set pane1 [$pw1 add -minsize 100 -weight 10]
    set pane2 [$pw1 add -minsize 40 -weight 1]

    canvas $pane1.c \
        -width 295 \
        -height 257 \
        -background #CCCCCC \
        -borderwidth 2 \
        -relief ridge \
        -takefocus 0 \
        -yscrollcommand {.pgaw:VisualQuery.pw.f0.frame.sy set} \
        -xscrollcommand {.pgaw:VisualQuery.pw.f0.frame.sx set} \
        -scrollregion {-297.0c -210.0c 297.0c 210.0c}

    scrollbar $pane1.sy \
        -orient vertical \
        -command [list $pane1.c yview]
    scrollbar $pane1.sx \
        -orient horizontal \
        -command [list $pane1.c xview]

    grid $pane1.c \
        -sticky news \
        -row 0 \
        -column 0
    grid $pane1.sy \
        -sticky ns \
        -row 0 \
        -column 1
    grid $pane1.sx \
        -sticky ew \
        -row 1 \
        -column 0
    grid rowconfigure $pane1 0 \
        -weight 10
    grid columnconfigure $pane1 0 \
        -weight 10

    tablelist::addBWidgetComboBox
    set VisualQB::Win(tbl) [tablelist::tablelist $pane2.tb \
        -font $PgAcVar(pref,font_fix) \
        -selectmode extended \
        -stretch all \
        -takefocus 1 \
        -selectbackground #DDDDDD \
        -selectforeground #708090 \
        -showarrow 1 \
        -editstartcommand VisualQB::startFieldEdit \
        -yscrollcommand [list $pane2.sy set] \
        -xscrollcommand [list $pane2.sx set]]

    scrollbar $pane2.sy \
        -orient vertical \
        -command [list $pane2.tb yview]
    scrollbar $pane2.sx \
        -orient horizontal \
        -command [list $pane2.tb xview]

    grid $pane2.tb \
        -sticky news \
        -row 0 \
        -column 0
    grid $pane2.sy \
        -sticky ns \
        -row 0 \
        -column 1
    grid $pane2.sx \
        -sticky ew \
        -row 1 \
        -column 0
    grid rowconfigure $pane2 0 \
        -weight 10
    grid columnconfigure $pane2 0 \
        -weight 10

    for {set c 1} {$c < [$pane2.tb cget -columns]} {incr c} {
        set cb [checkbutton $pane2.tb.cb$c \
            -bg white \
            -activebackground white]
        $pane2.tb window config 6,$c \
            -window $cb \
            -padx 1 \
            -pady 1 \
            -sticky news
    }

    # Set table properties
    #$pane2.tb tag config title -bg #CCCCCC -fg #000000 -anchor e
    #$pane2.tb height 0 -10
    #$pane2.tb width 0 -78
    #$pane2.tb tag configure sel -fg black
    #$pane2.tb tag configure active -fg black

    # set up tags for the various states of the columns
    #$pane2.tb tag configure HILITEcol -bg darkblue -fg white
    #$pane2.tb tag configure HILITErow -bg darkblue -fg white

    #place $pane2.tb -x 0 -y 0 -relheight 1.0 -relwidth 1.0 -height -18 -width -18 -anchor nw
    #place $pane2.sy -relx 1.0 -y 0 -relheight 1.0 -height -18 -width 18 -anchor ne
    #place $pane2.sx -x -18 -relx 1.0 -rely 1.0 -relwidth 1.0 -width [expr [$pane2.tb width 0] + -18] -anchor se



    # ------------- Menus --------------------------
    # create popup menu for bottom table grid
    set p $pane2.tb.pop
    menu $p -type normal
    $p add command \
        -label "Delete" \
        -command ::VisualQB::deleteObject

    # create popup menu for the top canvas portion
    set lnk $pane1.c.pop
    menu $lnk -type normal
    $lnk add cascade \
        -label "Join Type..." \
        -underline 0 \
        -command ::VisualQB::editJoinType
    $lnk add cascade \
        -label "Delete" \
        -underline 0 \
        -command ::VisualQB::deleteObject
    # ------------ End Menus -----------------------


    # display widgets
    # frame
    pack $base.fb \
    -in $base \
    -expand 0 \
    -fill x
    # label
    pack $base.fb.ltable \
    -in $base.fb \
    -side left
    # combobox
    pack $base.fb.cbtable \
    -in $base.fb \
    -side left
    # toolbar
    pack $base.fb.bbox \
    -in $base.fb \
    -side right \
    -expand 0 \
    -fill x
    # PanedWindow
    pack $pw1 \
    -in $base \
    -expand 1 \
    -fill both

    # point to canvas
    set Win(cv) .pgaw:VisualQuery.pw.f0.frame.c

    # some helpful key bindings for canvas
    bind $Win(cv) <Control-Key-w> [subst {destroy $base}]
    bind $Win(cv) <Key-Delete> {VisualQB::deleteObject}
    bind $Win(cv) <Button-1> {VisualQB::canvasClick %x %y}
    bind $Win(cv) <Button-3> {VisualQB::canvasButton3 %W %x %y}

    # point to table, only need to do this once
    set Win(tbl) .pgaw:VisualQuery.pw.f1.frame.tb

    # some helpful key bindings for table
    bind [$Win(tbl) bodytag] <Button-1> {VisualQB::tableButton1 %W %x %y}
    bind [$Win(tbl) bodytag] <Button-3> {VisualQB::tableButton3 %W %x %y}

    # Set up events for drop on table
    # -dropcmd and -droptypes must be set for DropSite to work
    foreach w [winfo children $Win(tbl)] {
        DropSite::register $w \
            -dropcmd ::VisualQB::tableDrop \
            -droptypes [list LISTBOX_ITEM [list copy]]
    }
}; # end proc vTclWindow.pgaw:VisualQuery
