#----------------------------------------------------------
# config.tcl
#
#    Parsing the main configuration files
#
#----------------------------------------------------------
debug "\nParsing the configuration file:"

# PGACCESS_DIR - the directory for the config files
# setting the variable...
if {![info exists PgAcVar(PGACCESS_DIR)]} {
	set PgAcVar(PGACCESS_DIR) [file join $PgAcVar(HOME) .pgaccess]
}
# and creating the directory if needed
if {![file isdirectory $PgAcVar(PGACCESS_DIR)]} {
	debug "\tCreating $PgAcVar(PGACCESS_DIR) ..."
	if {[catch {file mkdir $PgAcVar(PGACCESS_DIR)} msg]} {
		debug "\tWarning: the $PgAcVar(PGACCESS_DIR) directory"
		debug "\tfor storing the config stuff cannot be created ($msg)."
	}
}

# PGACCESS_CFG - the main config file
# setting the variable...
if {![info exists PgAcVar(PGACCESS_CFG)]} {
	set PgAcVar(PGACCESS_CFG) [file join $PgAcVar(PGACCESS_DIR) pgaccess.cfg]
}
if {[file exists $PgAcVar(PGACCESS_CFG)]} {
	# parsing the main config file
	debug "\nLoading the '$PgAcVar(PGACCESS_CFG)' config file:"
	set fid [open $PgAcVar(PGACCESS_CFG) r]
	if {$fid == ""} {
		return 0
	}
	set line_no 0
	while {![eof $fid]} {
		incr line_no
		catch {
			set line [string trim [gets $fid]]
			if {![string equal -length 1 $line "#"] && ![string equal $line ""]} {
				if {[llength [split $line "="]] == 2} {
					set var [string trim [lindex [split $line "="] 0]]
					set value [string trim [lindex [split $line "="] 1]]
					if {$var != ""} {
						set PgAcVar($var) $value
						debug "\t$var: $value"
					} else {
						debug "\tSyntax error ($filename,$line_no): null variable name"
					}
				} else {
					debug "\tSyntax error ($filename,$line_no): too many tokens"
				}
			}
		}
	}
	close $fid
} elseif {[file isdirectory $PgAcVar(PGACCESS_DIR)]} {
	# or creating it if needed
	debug "\Creating $PgAcVar(PGACCESS_CFG) ..."
	if {[catch {
		set fid [open $PgAcVar(PGACCESS_CFG) w]
		puts $fid "# PgAccess main config file"
		puts $fid ""
		puts $fid "# path to libpgtcl"
		if {[info exists PgAcVar(PGLIB)]} {
			puts $fid "# PGLIB = $PgAcVar(PGLIB)"
		} else {
			puts $fid "# PGLIB = /usr/lib"
		}
		puts $fid ""
		puts $fid "# script path"
		if {[info exists PgAcVar(PGACCESS_HOME)]} {
			puts $fid "# PGACCESS_HOME = $PgAcVar(PGACCESS_HOME)"
		} else {
			puts $fid "# PGACCESS_HOME = /usr/lib/pgaccess"
		}
		puts $fid ""
		close $fid
	} msg]} {
		debug "\tWarning: the $PgAcVar(PGACCESS_CFG) config file cannot be created ($msg)."
	}
}

# PGACCESS_RC - the rc file
if {![info exists PgAcVar(PGACCESS_RC)]} {
	set PgAcVar(PGACCESS_RC) [file join $PgAcVar(PGACCESS_DIR) pgaccessrc]
}

# PGACCESS_CONN - th connections file
if {![info exists PgAcVar(PGACCESS_CONN)]} {
	set PgAcVar(PGACCESS_CONN) [file join $PgAcVar(PGACCESS_DIR) connections]
}
