#----------------------------------------------------------
# defaults.tcl
#
#    Sets the defaults for various settings
#
#----------------------------------------------------------

debug "\nSetting the default values:"

# HOME directory of the user
set PgAcVar(HOME) ""
debug "\tPgAcVar(HOME): $PgAcVar(HOME)"

# documents directory
set PgAcVar(PGACCESS_DOC) [file join $PgAcVar(PGACCESS_HOME) doc]
debug "\tPgAcVar(PGACCESS_DOC): $PgAcVar(PGACCESS_DOC)"

# hostname of the actual database
set PgAcVar(currentdb,host) ""
debug "\tPgAcVar(currentdb,host): $PgAcVar(currentdb,host)"

#
set PgAcVar(currentdb,pgport) 5432
debug "\tPgAcVar(currentdb,pgport): $PgAcVar(currentdb,pgport)"

#
set CurrentDB ""
debug "\tCurrentDB: $CurrentDB"

# all the namespaces
set PgAcVar(namespaces) {
	colors mainlib syntax database debug
	tables queries visualqb forms views functions
	reports scripts usergroups sequences diagrams help
	preferences printer importexport connections graphs
	pgackages images types domains triggers indexes rules
	languages casts operators operatorclasses aggregates
	conversions newdbwiz icons
}
debug "\tPgAcVar(namespaces): $PgAcVar(namespaces)"

#
set PgAcVar(tablist) {
	Tables Queries Views Sequences Functions Reports Graphs
	Forms Scripts Images Usergroups Diagrams Types Domains
	Triggers Indexes Rules Languages Casts Operators
	Operatorclasses Aggregates Conversions
}
debug "\tPgAcVar(tablist): $PgAcVar(tablist)"

#
set PgAcVar(pgtablist) {
	Tables Indexes Views Rules Sequences Functions Triggers
	Aggregates Types Domains Usergroups Languages Conversions
	Casts Operators Operatorclasses
}
debug "\tPgAcVar(pgtablist): $PgAcVar(pgtablist)"

#
set PgAcVar(pgatablist) {Queries Reports Graphs Forms Scripts Images Diagrams}
debug "\tPgAcVar(pgatablist): $PgAcVar(pgatablist)"

#
set PgAcVar(pgtypes) {
	char varchar text int2 int4 serial float4 float8 money
	abstime date datetime interval reltime time timespan timestamp
	boolean box circle line lseg path point polygon
}
debug "\tPgAcVar(pgtypes): $PgAcVar(pgtypes)"

# the following are all defined in main/cmd_line.tcl

#
set PgAcVar(activetab) {}
debug "\tPgAcVar(activetab): $PgAcVar(activetab)"

#
set PgAcVar(query,tables) {}
debug "\tPgAcVar(query,tables): $PgAcVar(query,tables)"

#
set PgAcVar(query,links) {}
debug "\tPgAcVar(query,links): $PgAcVar(query,links)"

#
set PgAcVar(query,results) {}
debug "\tPgAcVar(query,results): $PgAcVar(query,results)"

#
set PgAcVar(mwcount) 0
debug "\tPgAcVar(mwcount): $PgAcVar(mwcount)"

#
set PgAcVar(PGACCESS_LOGIN) 0
debug "\tPgAcVar(PGACCESS_LOGIN): $PgAcVar(PGACCESS_LOGIN)"

#
set PgAcVar(PGACCESS_SMLOGIN) 0
debug "\tPgAcVar(PGACCESS_SMLOGIN): $PgAcVar(PGACCESS_SMLOGIN)"

#
set PgAcVar(PGACCESS_NOAUTO) 0
debug "\tPgAcVar(PGACCESS_NOAUTO): $PgAcVar(PGACCESS_NOAUTO)"

#
set PgAcVar(PGACCESS_NOCONN) 0
debug "\tPgAcVar(PGACCESS_NOCONN): $PgAcVar(PGACCESS_NOCONN)"

#
set PgAcVar(PGACCESS_NOSCRIPT) 0
debug "\tPgAcVar(PGACCESS_NOSCRIPT): $PgAcVar(PGACCESS_NOSCRIPT)"

#
set PgAcVar(PGACCESS_CONNLOAD) 0
debug "\tPgAcVar(PGACCESS_CONNLOAD): $PgAcVar(PGACCESS_CONNLOAD)"

#
set PgAcVar(PGACCESS_HIDDEN) 0
debug "\tPgAcVar(PGACCESS_HIDDEN): $PgAcVar(PGACCESS_HIDDEN)"

#
set PgAcVar(PGACCESS_NOSPLASH) 0
debug "\tPgAcVar(PGACCESS_NOSPLASH): $PgAcVar(PGACCESS_NOSPLASH)"

#
set PgAcVar(opendb,host) "localhost"
debug "\tPgAcVar(opendb,host): $PgAcVar(opendb,host)"

#
set PgAcVar(opendb,pgport) "5432"
debug "\tPgAcVar(opendb,pgport): $PgAcVar(opendb,pgport)"

#
set PgAcVar(opendb,dbname) ""
debug "\tPgAcVar(opendb,dbname): $PgAcVar(opendb,dbname)"

#
set PgAcVar(opendb,username) ""
debug "\tPgAcVar(opendb,username): $PgAcVar(opendb,username)"

#
set PgAcVar(opendb,password) ""
debug "\tPgAcVar(opendb,password): $PgAcVar(opendb,password)"

# path to the libpgtcl shared object lib.
# Windows XP is seen as NT, but the Windows directory is called 'windows' and not 'winnt' -
# change the first switch according to the Windows version you run. (Iavor)
# also make sure that we arent using PGINTCL
switch -- $::tcl_platform(platform) {
	windows {
		switch -- $::tcl_platform(os) {
			"Windows NT" {
				set PgAcVar(PGLIB) "c:/windows/system32/"
			}
			"Windows 95" {
				set PgAcVar(PGLIB) "c:/windows/system/"
			}
			default {
				set PgAcVar(PGLIB) ""
			}
		}
	}
	unix {
		set PgAcVar(PGLIB) "/usr/lib"
	}
	MacOS {
		# How is this on Macintosh platform?
		set PgAcVar(PGLIB) ""
	}
	default {
		# for all other cases
		set PgAcVar(PGLIB) ""
	}
}
debug "\tPgAcVar(PGLIB): $PgAcVar(PGLIB)"
