#----------------------------------------------------------
# util.tcl
#
#    Utility functions
#
#----------------------------------------------------------

#----------------------------------------------------------
# parameter --
#
#    ???
#
# Arguments:
#    none
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc parameter {msg} {
    global PgAcVar
    Window show .pgaw:GetParameter
    focus .pgaw:GetParameter.e1
    set PgAcVar(getqueryparam,var) ""
    set PgAcVar(getqueryparam,flag) 0
    set PgAcVar(getqueryparam,msg) $msg
    bind .pgaw:GetParameter <Destroy> "set PgAcVar(getqueryparam,flag) 1"
    grab .pgaw:GetParameter
    tkwait variable PgAcVar(getqueryparam,flag)
    if {$PgAcVar(getqueryparam,result)} {
        return $PgAcVar(getqueryparam,var)
    } else {
        return ""
    }
}

#----------------------------------------------------------
# sql_exec --
#
#   executes a line of sql
#
# Parameters:
#   how_   the execution mode, quiet - no error message on fail
#   cmd_   the sql command
#   dbh_   the database handle (optional)
#   win_   parent window we display error in, if any (optional)
#
# Results:
#   none
#----------------------------------------------------------
#
proc sql_exec {how_ cmd_ {dbh_ ""} {win_ ""}} {

    global PgAcVar CurrentDB

    if {[string match "" $dbh_]} {
        set dbh_ $CurrentDB
    }

    if {[set pgr [wpg_exec $dbh_ $cmd_]]==0} {
        return 0
    }

    if {$PgAcVar(pgsql,status) == "PGRES_COMMAND_OK"
     || $PgAcVar(pgsql,status) == "PGRES_TUPLES_OK"} {
        pg_result $pgr -clear
        return 1
    }

    if {$how_ != "quiet"} {
        showError [format [intlmsg "Error executing query\n\n%s\n\nPostgreSQL error message:\n%s\nPostgreSQL status:%s"] $cmd_ $PgAcVar(pgsql,errmsg) $PgAcVar(pgsql,status)] {} $win_
    }

    pg_result $pgr -clear

    return 0
}

#----------------------------------------------------------
# PgAcVar:clean --
#
#    Cleans out a section of the PgAcVar array
#
# Arguments:
#    prefix    the section of the array that needs
#              to be deleted
#
# Results:
#
#----------------------------------------------------------
#
proc PgAcVar:clean {prefix} {
    global PgAcVar
    foreach key [array names PgAcVar $prefix] {
        set PgAcVar($key) {}
        unset PgAcVar($key)
    }
}

#----------------------------------------------------------
# intlmsg --
#
#    handles displaying the correct message based on the
#    language in use
#
# Arguments:
#    msg    the message key to display
#
# Results:
#
#----------------------------------------------------------
#
proc intlmsg {msg} {
    global PgAcVar Messages
    if {$PgAcVar(pref,language)=="english"} { return $msg }
    if { ! [array exists Messages] } { return $msg }
    if { ! [info exists Messages($msg)] } {
        debug "Please translate: $msg"
        return $msg
    }
    return $Messages($msg)
}


#----------------------------------------------------------
# showError --
#
#    show the error
#    provide a link to help if one was specified
#
# Arguments:
#    emsg_          error message
#    helptopic_     help topic to reference (optional)
#    win_           parent window of dialog (optional)
#
# Results:
#   none
#----------------------------------------------------------
#
proc showError {emsg_ {helptopic_ {}} {win_ {}}} {
    global PgAcVar
    set blist [list "OK" "Help"]
    bell
    debug $emsg_

    # TODO fix this font stuff
    set max [expr {40*[font measure {Helvetica 11} h]}]
    foreach L [split $emsg_ \n] {
        set len [expr {40*[string length $L]}]
        if {$len > $max} {
            set max $len
        }
    }

    if {$PgAcVar(pref,erroremailuse)} {
        lappend blist "Report"
    }

    set res [MessageDlg $win_.pgacErrorDlg \
        -title [intlmsg "Error"] \
        -anchor e \
        -width $max \
        -icon error \
        -message $emsg_ \
        -type user \
        -buttons $blist]

    switch -- $res {
        0 {
            # they hit ok, just close the dialog
        }
        1 {
            # they need help
            if {[string length $helptopic]>0} {
                Help::load $helptopic
            } else {
                Help::load index
            }
        }
        2 {
            # they want to report the error
            tk_messageBox \
            -message "Soon you might be able to email bug reports.
            But not yet."
        }
        default {
            # what did they hit
            debug "You should never see this."
        }
    }; # end switch

}; # end proc showError


#----------------------------------------------------------
# say --
#
#    displays message a in message box
#
# Argument: msg - message to display
#----------------------------------------------------------
#
proc say {msg} {
    tk_messageBox -message $msg
}

#----------------------------------------------------------
# debug --
#
#    displays debug messages on stdout
#
# Argument: msg - debug message to display
#----------------------------------------------------------
#
proc debug {msg} {
    global PgAcVar
    if {![info exists PgAcVar(DEBUG)]} {
        set PgAcVar(DEBUG) false
    }
    if {![info exists PgAcVar(DEBUG_STR)]} {
        set PgAcVar(DEBUG_STR) ""
    }
    if {$PgAcVar(DEBUG_STR) == ""} {
        set PgAcVar(DEBUG_STR) "$msg"
    } else {
        set PgAcVar(DEBUG_STR) "$PgAcVar(DEBUG_STR)\n$msg"
    }
    if {$PgAcVar(DEBUG)} {
        if {[winfo exists .pgaw:Splash]} {
            foreach ln [split $PgAcVar(DEBUG_STR) "\n"] {
                after 50
                if {![string match "\t*" $ln] && ( ![info exists PgAcVar(PGACCESS_NOSPLASH)] ) } {
                    splash_puts [string range [string trim [string trim $ln] ":"] 0 30]
                }
            }
        }
        puts $PgAcVar(DEBUG_STR)
        set PgAcVar(DEBUG_STR) ""
    }
}
