#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"

wm withdraw .
update idletasks

# getting the script's main directory
if {[lsearch -exact $argv "-home"] != -1} {
	# as command line parameter
	set PgAcVar(PGACCESS_HOME) [lindex $argv [expr [lsearch -exact $argv "-home"] + 1]]
	if {![file isdirectory $PgAcVar(PGACCESS_HOME)]} {
		puts stderr "ERROR: '$PgAcVar(PGACCESS_HOME)' Not valid directory."
		puts stderr "Please check your '-home' command line parameter."
	}
} elseif {[info exists env(PGACCESS_HOME)]} {
	# as environment variable
	set PgAcVar(PGACCESS_HOME) $env(PGACCESS_HOME)
	if {![file isdirectory $PgAcVar(PGACCESS_HOME)]} {
		puts stderr "ERROR: '$PgAcVar(PGACCESS_HOME)' Not valid directory."
		puts stderr "Please check your 'PGACCESS_HOME' environment setting."
	}
} else {
	# none of the above
	set tmphome [info script]
	if {[file type $tmphome] == "link"} {
		set home [file dirname [file readlink $tmphome]]
	} else {
		set home [file dirname [info script]]
	}
	switch -- [file pathtype $home] {
		absolute {set PgAcVar(PGACCESS_HOME) $home}
		relative {set PgAcVar(PGACCESS_HOME) [file join [pwd] $home]}
		volumerelative {
			set curdir [pwd]
			cd $home
			set PgAcVar(PGACCESS_HOME) [file join [pwd] [file dirname [file join [lrange [file split $home] 1 end]]]]
			cd $curdir
		}
	}
	unset tmphome
	unset home
}

#loading the main modules
if {[catch {

	# PgAccess versioning
	set PgAcVar(MAIN_VERSION) "1.0"
	#set PgAcVar(LAST_BUILD) "20040219"
	set PgAcVar(LAST_BUILD) "pre-alpha"
	source [file join $PgAcVar(PGACCESS_HOME) main version.tcl]
	set PgAcVar(VERSION) [get_version]

	# loading window handling routines
	source [file join $PgAcVar(PGACCESS_HOME) main window.tcl]

	# loading window handling routines
	source [file join $PgAcVar(PGACCESS_HOME) main splash.tcl]

	# loading utility functions
	source [file join $PgAcVar(PGACCESS_HOME) main util.tcl]
	
	# initialising the variables
	source [file join $PgAcVar(PGACCESS_HOME) main defaults.tcl]
	
	# loading the environment variables
	source [file join $PgAcVar(PGACCESS_HOME) main env.tcl]
	
	# parsing the config file
	source [file join $PgAcVar(PGACCESS_HOME) main config.tcl]
	
	# parsing the command line parameters
	source [file join $PgAcVar(PGACCESS_HOME) main cmd_line.tcl]

	# loading plugin/namespace/package handling routines
	source [file join $PgAcVar(PGACCESS_HOME) main load.tcl]

} msg]} {
	puts stderr "ERROR: $msg"
	puts stderr "Please make sure that PgAccess is installed correctly in the '$PgAcVar(PGACCESS_HOME)' directory."
	exit 1
}

if {!$PgAcVar(PGACCESS_NOSPLASH)} {
    set splash_clock [clock seconds]
    Window show .pgaw:Splash
}

splash_percent 10
debug "\nLoading the Tcl-PostgreSQL interface:"
# only load PGLIB if PGINTCL not specified on command line
if { ![info exists PgAcVar(PGINTCL)] } {
	debug "\tCouldn't use pgin.tcl, the pure Tcl interface."
	# now that pgtcl is a proper tcl package, try it
	if {[catch {package require Pgtcl} msg]} {
		debug "\tCouldn't use the new Pgtcl package."
		set shlib [file join $PgAcVar(PGLIB) libpgtcl][info sharedlibextension]
		if {![file exists $shlib]} {
			debug "\tError: Shared library file: '$shlib' does not exist."
			debug "\tCheck this file, or check PGLIB variable (in pgaccess.cfg)"
			debug "\t====> Using pgin.tcl instead"
			set PgAcVar(PGINTCL) 1
		}
		
		if {[catch {load $shlib} err]} {
			debug "\tError: can not load $shlib shared library."
			debug "\tYou need to make sure that the library exists and"
			debug "\tthe environment variable PGLIB points to the directory"
			debug "\twhere it is located.\n"
			debug "\tIf you use Windows, be sure that the needed libpgtcl.dll"
			debug "\tand libpq.dll files are copied in the Windows/System"
			debug "\tdirectory.\n"
			debug "\tERROR MESSAGE: $err"
			debug "\t====> Using pgin.tcl instead"
			set PgAcVar(PGINTCL) 1
		} else {
			debug "\tLoaded the old libpgtcl shared library."
		}
	} else {
		debug "\tLoaded the new Pgtcl package."
	}; # end check of package require pgtcl
} else {
	debug "\tWill try loading the pgin.tcl plugin soon."
}

# loading namespaces
splash_percent 20
init_load_namespaces

# loading plugins
splash_percent 50
init_load_plugins

# loading preferences
splash_percent 60
Preferences::load

# loading connections
splash_percent 70
Connections::load

# loading required_packages
splash_percent 80
init_load_required_packages

# Creating icons
splash_percent 90
debug "\nCreating icons:"
set idir [file join $PgAcVar(PGACCESS_HOME) lib widgets icons1.0]

::icons::icons create \
-file [file join $idir tkIcons-sample.slick] -group *

::icons::icons create \
-file [file join $idir tkIcons] {actcheck16 actcross16 editdelete-22}

# load the style
splash_percent 95
set sfile [file join $PgAcVar(PGACCESS_HOME) lib style pg_style.tcl]
if {[file exists $sfile]} {
	source $sfile
	package require pga::style
	pga::style::init
}

# load the msgcat files
::msgcat::mcload [file join $PgAcVar(PGACCESS_HOME) lib languages]

debug "\nCreating the main window..."
update idletasks
splash_percent 97

Window show .pgaw:Main

#####################################################################

debug "\nLoading the saved connections..."
update idletasks
splash_percent 99

set tdb ""
set thost ""
set id ""

if {[info exists PgAcVar(pref,lastdb)] \
&& [info exists PgAcVar(pref,lasthost)]} {
	set tdb $PgAcVar(pref,lastdb)
	set thost $PgAcVar(pref,lasthost)
}
##
## Check to see if connections should be opened automagically
##
if {$PgAcVar(pref,autoload)
&& !$PgAcVar(PGACCESS_NOAUTO)
&& !$PgAcVar(PGACCESS_NOCONN)} {
	##
	##  Load all dbs that are in the connections file
	##
	foreach H [::Connections::getHosts] {
		
		foreach d [::Connections::getDbs $H] {
			
			set i [::Connections::getIds $H $d]
			
			if {[llength $i] != 1} {
				puts stderr "\nERROR: There seems to be a problem with your connections file."
				puts stderr "A host/db combination should be unique and the db should not be empty string"
				puts stderr "Check host/db:  $H/$d with ids: $i"
				puts stderr "Try removing the ~/.pgaccess/connections file"
				puts stderr "Skipping this host/db combination\n"
				continue
			}
			
			
			##
			##    This is just to make sure
			##    the connections variables
			##    are all set from older versions
			##
			foreach {S C} {
				autoload autoload
				systemtables viewsystem
				pgaccesstables viewpgaccess
				savepasswords savepasswords
			} {
				
				if {![info exists ::Connections::Conn($C,$i)]} {
					set ::Connections::Conn($C,$i) $PgAcVar(pref,$S)
				}
			}
			
			if {$::Connections::Conn(autoload,$i) == 0} {
				continue
			}
			
			if {![::Connections::openConn $i]} {
				debug "$::Connections::Msg($i)"
				# display the open conn window
				#::Connections::openConnWin $i 1
				#wm withdraw .pgaw:Splash
			} else {
				
				set id $i
				::Mainlib::addDbNode $H $d $id
			}
			
				set id $i
		}; # end foreach DB
		
	}; # end foreach host
	
	#Connections::check
	
	##
	##  Open up the tree node for the last db
	##
	if {[string match "" $thost]} {set thost sockets}
	
	if {(![string match "" $tdb]) && ([$::Mainlib::Win(tree) exists db-$id])} {
		
		if {[$::Mainlib::Win(tree) exists host-$id]} {
			$::Mainlib::Win(tree) opentree host-$id 0
		}
		if {[$::Mainlib::Win(tree) exists db-${id}]} {
			$::Mainlib::Win(tree) opentree db-${id}
			::Mainlib::select 1 db-${id}
		}
	}
	
}; # end if autoload (or if NO_AUTO doesnt exist)

# checking if we should connect based on the given command line parameters
# this looks a little gross, but some modules rely on the GUI a bit too much
if {$PgAcVar(PGACCESS_CONNLOAD)} {
	if {[::Connections::openConn 0 $PgAcVar(PGACCESS_LOGIN) $PgAcVar(PGACCESS_SMLOGIN)]} {
		set H $PgAcVar(opendb,host)
		set d $PgAcVar(opendb,dbname)
		set id [lindex [::Connections::getIds $H $d] 0]
		if {![$::Mainlib::Win(tree) exists db-${id}]} {
			::Mainlib::addDbNode $H $d $id
		}
		if {[$::Mainlib::Win(tree) exists host-$id]} {
			$::Mainlib::Win(tree) opentree host-$id 0
		}
		if {[$::Mainlib::Win(tree) exists db-${id}]} {
			$::Mainlib::Win(tree) opentree db-${id}
		}
		if {[$::Mainlib::Win(tree) exists host-$id]} {
			::Mainlib::select 1 host-$iid
		}
		if {[$::Mainlib::Win(tree) exists db-${id}]} {
			::Mainlib::select 1 db-${id}
		}
		# ::Mainlib::cmd_Tables
	} else {
        if { [winfo exists .pgaw:Splash] } {
            wm withdraw .pgaw:Splash
        }
		showError [intlmsg "Cannot login, possible bad connection parameters"]
	}
} elseif {$PgAcVar(PGACCESS_LOGIN)} {
    foreach {k v} [array get PgAcVar(opendb,*)] {puts "$k : $v"}
	::Connections::openConn 0 $PgAcVar(PGACCESS_LOGIN) $PgAcVar(PGACCESS_SMLOGIN)
} else {
    # nothing
}

###############################################################################

debug "\nOpening the main window..."
update idletasks
splash_percent 100

# let's wait until the bar is full before we stop splashing
if {!$PgAcVar(PGACCESS_NOSPLASH)} {
    # splash screen shown at least for 2 seconds
    while {[expr {[clock seconds] - $splash_clock}] < 2} {
        after 100
    }
    wm withdraw .pgaw:Splash
}

# show the main window
if {!$PgAcVar(PGACCESS_HIDDEN)} {
	wm deiconify .
}

