#! /bin/sh
# \
exec tclsh "$0" ${1+"$@"}

#--------------------------------------------------------------------
# Args
#    pgaccess dir: this is where the pgaccess distribution
#                  is located
#    root dest dir: the temp dir where we will copy
#                   the pgaccess code to build the star(kit/pack)
#    -starpack dir: create a starpack instead of a starkit, and
#                   specify the tclkit to use (needs to be different
#                   than what is the active tclkit
#--------------------------------------------------------------------
if {($argc != 2) && ($argc != 4)} {
    puts "\nWrong args: $argv0 <pgaccess dir> <root destination dir> ?-starpack /path/to/tclkit?\n"
    puts "For example: $argv0 /path/to/pgaccess /tmp\n"
    exit
}

set dir(src) [lindex $argv 0]
if {[string equal "." $dir(src)]} {set dir(src) [pwd]}
set dir(win32) [file join $dir(src) op_sys win32]
set dir(root) [lindex $argv 1]
set dir(dest) [file join $dir(root) pgaccess.vfs]
set dir(pgtcl) [file join $dir(root) pgaccess.vfs lib pgtcl]
set file(star) pgaccess.kit
set ico 0
set file(main) [file join $dir(dest) main.tcl]

set starkit ""
if {($argc == 4) && ([string match "-star*" [lindex $argv 2]])} {
    puts "BUILDING A STARPACK"
    set starkit "-runtime [lindex $argv 3]"
    if {[string match "*win*" [lindex $argv 3]]} {
        set file(star) pgaccess.exe
        set ico 1
    } else {
        set file(star) pgaccess.bin
    }
} else {
    puts "BUILDING A STARKIT"
}

if {[file exists $dir(dest)]} {

    puts "Deleting existing vfs directory ($dir(dest)) ..."
    if {[catch {file delete -force $dir(dest)} err]} {
        puts "Error while deleting $dir(dest)...make sure you have permissions"
        puts "Error msg: $err"
        exit
    }
}

puts "Making vfs directory..."
if {[catch {file mkdir $dir(dest)} err]} {
        puts "Error while making dir $dir(dest)..."
        puts "Error msg: $err"
        exit
}

puts "Copying files from src dir ($dir(src)) to vfs directory ($dir(dest)) ..."
foreach F [glob -dir $dir(src) *] {
    if {![string match "*op_sys" $F]} {
        puts "FILE: $F"
        if {[catch {file copy $F $dir(dest)} err]} {
            puts "Error while copying files from $dir(src) to $dir(dest)..."
            puts "Error msg: $err"
            exit
        }
    }
}

# check if windows icon is needed
if {$ico} {
    if {[catch {file copy $dir(dest)/images/tclkit.ico $dir(dest)}]} {
        puts "Error creating icon, will use default instead."
    }
}

# need to make pgtcl dir for Win DLL
if {[catch {file mkdir $dir(pgtcl)} err]} {
        puts "Error while making dir $dir(pgtcl)..."
        puts "Error msg: $err"
        exit
}

foreach F {libpgtcl1.4.dll pkgIndex.tcl} {
    if {[catch {file copy -force [file join $dir(win32) dll $F] $dir(pgtcl)} err]} {
        puts "Error while copying files from $dir(src) to $dir(dest)..."
        puts "Error msg: $err"
        exit
    }
}

set contents {package require starkit
package require Tk
starkit::startup
#package provide tcllib
set env(PGACCESS_HOME) [file join $starkit::topdir]
starkit::autoextend [file join $env(PGACCESS_HOME) main]
starkit::autoextend [file join $env(PGACCESS_HOME) images]
starkit::autoextend [file join $env(PGACCESS_HOME) lib]
starkit::autoextend [file join $env(PGACCESS_HOME) lib help]
starkit::autoextend [file join $env(PGACCESS_HOME) lib widgets]
starkit::autoextend [file join $env(PGACCESS_HOME) extra]
source [file join $starkit::topdir pgaccess.tcl]}

puts "Writing main.tcl ... "
set fid [open $file(main) w]
puts $fid $contents
close $fid

puts "Creating $file(star) ... "
set PWD [pwd]
cd $dir(root)
puts [eval exec sdx.kit wrap $file(star) $starkit]
cd $PWD
